@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # Login Screen
 #
-# $Id: lonlogin.pm,v 1.94.2.1 2007/10/09 16:48:51 albertel Exp $
+# $Id: lonlogin.pm,v 1.97 2007/10/09 16:27:08 albertel Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -30,7 +30,6 @@ package Apache::lonlogin;
 
 use strict;
 use Apache::Constants qw(:common);
-use CGI::Cookie();
 use Apache::File ();
 use Apache::lonnet;
 use Apache::loncommon();
@@ -69,24 +68,17 @@ sub handler {
 
 
 # -------------------------------- Prevent users from attempting to login twice
-    my %cookies=CGI::Cookie->parse($r->header_in('Cookie'));
-    my $lonid=$cookies{'lonID'};
-    my $cookie;
-    if ($lonid) {
-	my $handle=&LONCAPA::clean_handle($lonid->value);
-        my $lonidsdir=$r->dir_config('lonIDsDir');
-	if (-e "$lonidsdir/$handle.id") {
-# Is there an existing token file?
-	    if ($handle=~/^publicuser\_/) {
+    my $handle = &Apache::lonnet::check_for_valid_session($r);
+    if ($handle=~/^publicuser\_/) {
 # For "public user" - remove it, we apparently really want to login
-		unlink("$lonidsdir/$handle.id");
-	    } elsif ($handle ne '') {
+	unlink($r->dir_config('lonIDsDir')."/$handle.id");
+    } elsif ($handle ne '') {
 # Indeed, a valid token is found
-		my $start_page = 
-		    &Apache::loncommon::start_page('Already logged in');
-		my $end_page = 
-		    &Apache::loncommon::end_page();
-		$r->print(<<ENDFAILED);
+	my $start_page = 
+	    &Apache::loncommon::start_page('Already logged in');
+	my $end_page = 
+	    &Apache::loncommon::end_page();
+	$r->print(<<ENDFAILED);
 $start_page
 <h1>You are already logged in</h1>
 <p>Please either <a href="/adm/roles">continue the current session</a> or
@@ -95,9 +87,7 @@ $start_page
 <a href="/adm/loginproblems.html">Problems?</a></p>
 $end_page
 ENDFAILED
-                return OK;
- 	    }  
-	}
+        return OK;
     }
 
 # ---------------------------------------------------- No valid token, continue
