@@ -1,6 +1,6 @@
 # functions to glue school database system into Lon-CAPA for 
 # automated enrollment
-# $Id: localenroll.pm,v 1.24 2007/08/31 12:33:35 raeburn Exp $
+# $Id: localenroll.pm,v 1.27 2007/11/10 03:10:49 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -596,6 +596,111 @@ sub inst_usertypes {
     return $outcome;
 }
 
+###############################
+# sub username_rules
+#
+# Incoming data: three arguments 
+# (a) $dom - domain
+# (b) $ruleshash - reference to hash containing rules
+#                  (a hash of a hash)
+#                  keys of top level hash are short names  
+#                   (e.g., netid, noncredit) 
+#                  for each key, value is a hash
+#                      desc => long name for rule  
+#                      rule => description of rule
+#                      authtype => (krb5,krb4,int, or loc)
+#                                 authentication type for rule 
+#                      authparm => authentication parameter for rule
+#                      authparmfixed => 1 if authparm used when
+#                          creating user for rule must be authparm  
+#                      authmsg => Message to display describing 
+#                                 authentication to use for this rule
+#
+# (c) $rulesorder - reference to array containing rule names 
+#                   in order to be displayed
+
+#
+#  returns 'ok' if no processing error.
+#
+############################### 
+
+sub username_rules {
+    my ($dom,$ruleshash,$rulesorder) = @_;
+    my $outcome;
+    return $outcome;
+}
+
+###############################
+# sub id_rules
+#
+# Incoming data: three arguments
+# (a) $dom - domain
+# (b) $ruleshash - reference to hash containing rules
+#                  (a hash of a hash)
+#                  keys of top level hash are short names
+#                   (e.g., netid, noncredit)
+#                  for each key, value is a hash
+#                      desc => long name for rule
+#                      rule => description of rule
+#
+# (c) $rulesorder - reference to array containing rule names
+#                   in order to be displayed
+
+#
+#  returns 'ok' if no processing error.
+#
+###############################
+
+sub id_rules {
+    my ($dom,$ruleshash,$rulesorder) = @_;
+    my $outcome;
+    return $outcome;
+}
+
+###############################
+# sub username_check 
+#
+# Incoming data: four arguments
+# (a) $dom - domain (scalar) 
+# (b) $uname - username to compare against rules (scalar)
+# (c) $to_check (reference to array of rule names to check)
+# (d) $resultshash (reference to hash of results)
+#                    hash of results for rule checked
+#                   - keys are rule names
+#                   - values are: 1 or 0 (for matched or unmatched) 
+#
+# returns 'ok' if no processing error.
+#
+###############################
+
+sub username_check {
+    my ($dom,$uname,$to_check,$resultshash) = @_;
+    my $outcome;
+    return $outcome; 
+}
+
+###############################
+# sub id_check
+#
+# Incoming data: four arguments
+# (a) $dom - domain (scalar)
+# (b) $id - ID to compare against rules (scalar)
+# (c) $to_check (reference to array of rule names to check)
+# (d) $resultshash (reference to hash of results)
+#                    hash of results for rule checked
+#                   - keys are rule names
+#                   - values are: 1 or 0 (for matched or unmatched)
+#
+# returns 'ok' if no processing error.
+#
+###############################
+
+sub id_check {
+    my ($dom,$id,$to_check,$resultshash) = @_;
+    my $outcome;
+    return $outcome;
+}
+
 ###############################
 # sub AUTOLOAD
 #
