@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # The LON-CAPA Homework handler
 #
-# $Id: lonhomework.pm,v 1.268.2.2 2007/09/21 17:25:58 albertel Exp $
+# $Id: lonhomework.pm,v 1.286 2007/11/20 17:53:22 albertel Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -49,11 +49,11 @@ use Apache::matchresponse();
 use Apache::chemresponse();
 use Apache::drawimage();
 use Apache::Constants qw(:common);
-use HTML::Entities();
 use Apache::loncommon();
 use Apache::lonlocal;
 use Time::HiRes qw( gettimeofday tv_interval );
-use Apache::lonnet();
+use HTML::Entities();
+use File::Copy();
 
 # FIXME - improve commenting
 
@@ -63,6 +63,28 @@ BEGIN {
 }
 
 
+=pod
+
+=item set_bubble_lines()
+
+Called at analysis time to set the bubble lines
+hash for the problem.. This should be called in the
+end_problemtype tag in analysis mode.
+
+We fetch the hash of part id counters from lonxml
+    and push them into analyze:{part_id.bubble_lines}.
+
+=cut
+
+sub set_bubble_lines {
+    my %bubble_counters = &Apache::lonxml::get_bubble_line_hash();
+
+    foreach my $key (keys(%bubble_counters)) {
+	$Apache::lonhomework::analyze{"$key.bubble_lines"} =
+	    $bubble_counters{"$key"};
+    }
+}
+
 #
 # Decides what targets to render for.
 # Implicit inputs:
@@ -127,7 +149,7 @@ sub get_target {
 	    } elsif ( $env{'form.problemmode'} eq &mt('Edit') ||
 		      $env{'form.problemmode'} eq 'Edit') {
 		if ( $env{'form.submitted'} eq 'edit' ) {
-		    if ( $env{'form.submit'} eq &mt('Submit Changes and View') ) {
+		    if ( $env{'form.submit'} eq &mt('Save and View') ) {
 			return ('modified','web','answer');
 		    } else {
 			return ('modified','no_output_web','edit');
@@ -188,7 +210,6 @@ sub proctor_checked_in {
     return 0;
 }
 
-$Apache::lonxml::browse='';
 sub check_ip_acc {
     my ($acc)=@_;
     &Apache::lonxml::debug("acc is $acc");
@@ -377,8 +398,8 @@ sub check_access {
     if ($env{'request.state'} eq "construct") {
 	if ($env{'form.problemstate'}) {
 	    if ($env{'form.problemstate'} =~ /^CANNOT_ANSWER/) {
-		if ( ! ($env{'form.problemstate'} eq 'CANNOT_ANSWER_correct' &&
-			lc($Apache::lonhomework::problemstatus) eq 'no')) {
+		if ( ! ($env{'form.problemstate'} eq 'CANNOT_ANSWER_correct' 
+			&& &hide_problem_status())) {
 		    return ('CANNOT_ANSWER',
 			    &mt('is in this state due to author settings.'));
 		}
@@ -437,7 +458,7 @@ sub check_access {
 		$date = &mt("an indeterminate date"); $passed = 0;
 	    } else {
 		if (time < $date) { $passed = 0; } else { $passed = 1; }
-		$date = localtime $date;
+		$date = &Apache::lonlocal::locallocaltime($date);
 	    }
 	    if (!$passed) { $type=$temp; last; }
 	}
@@ -468,24 +489,28 @@ sub check_access {
 	# if (correct and show prob status) or excused then CANNOT_ANSWER
 	if(($Apache::lonhomework::history{"resource.$id.solved"}=~/^correct/
 	    &&
-	    lc($Apache::lonhomework::problemstatus) ne 'no')
+	    &show_problem_status())
 	   ||
 	   $Apache::lonhomework::history{"resource.$id.solved"}=~/^excused/) {
 	    $status = 'CANNOT_ANSWER';
 	}
+	if ($status eq 'CANNOT_ANSWER'
+	    && &show_answer_problem_status()) {
+	    $status = 'SHOW_ANSWER';
+	}
     }
     if ($status eq 'CAN_ANSWER' || $status eq 'CANNOT_ANSWER') {
-	my $interval=&Apache::lonnet::EXT("resource.$id.interval");
-	&Apache::lonxml::debug("looking for interval $interval");
-	if ($interval) {
-	    my $first_access=&Apache::lonnet::get_first_access('map');
+	my @interval=&Apache::lonnet::EXT("resource.$id.interval");
+	&Apache::lonxml::debug("looking for interval @interval");
+	if ($interval[0]) {
+	    my $first_access=&Apache::lonnet::get_first_access($interval[1]);
 	    &Apache::lonxml::debug("looking for accesstime $first_access");
 	    if (!$first_access) {
 		$status='NOT_YET_VIEWED';
 		my $due_date = &due_date($id);
 		my $seconds_left = $due_date - time;
-		if ($seconds_left > $interval || $due_date eq '') {
-		    $seconds_left = $interval;
+		if ($seconds_left > $interval[0] || $due_date eq '') {
+		    $seconds_left = $interval[0];
 		}
 		$datemsg=&seconds_to_human_length($seconds_left);
 	    }
@@ -511,18 +536,19 @@ sub check_access {
 sub due_date {
     my ($part_id,$symb,$udom,$uname)=@_;
     my $date;
-    my $interval= &Apache::lonnet::EXT("resource.$part_id.interval",$symb,
+    my @interval= &Apache::lonnet::EXT("resource.$part_id.interval",$symb,
 				       $udom,$uname);
-    &Apache::lonxml::debug("looking for interval $part_id $symb $interval");
+    &Apache::lonxml::debug("looking for interval $part_id $symb @interval");
     my $due_date= &Apache::lonnet::EXT("resource.$part_id.duedate",$symb,
 				       $udom,$uname);
     &Apache::lonxml::debug("looking for due_date $part_id $symb $due_date");
-    if ($interval =~ /\d+/) {
-	my $first_access=&Apache::lonnet::get_first_access('map',$symb);
-	&Apache::lonxml::debug("looking for first_access $first_access");
+    if ($interval[0] =~ /\d+/) {
+	my $first_access=&Apache::lonnet::get_first_access($interval[1],$symb);
+	&Apache::lonxml::debug("looking for first_access $first_access ($interval[1])");
 	if (defined($first_access)) {
-	    $interval = $first_access+$interval;
-	    $date = ($interval < $due_date)? $interval : $due_date;
+	    my $interval = $first_access+$interval[0];
+	    $date = (!$due_date || $interval < $due_date) ? $interval
+                                                          : $due_date;
 	} else {
 	    $date = $due_date;
 	}
@@ -662,9 +688,9 @@ sub handle_save_or_undo {
     my $error=0;
     if ($env{'form.Undo'} eq &mt('undo')) {
 	my $error=0;
-	if (!copy($file,$filetmp)) { $error=1; }
-	if ((!$error) && (!copy($filebak,$file))) { $error=1; }
-	if ((!$error) && (!move($filetmp,$filebak))) { $error=1; }
+	if (!&File::Copy::copy($file,$filetmp)) { $error=1; }
+	if ((!$error) && (!&File::Copy::copy($filebak,$file))) { $error=1; }
+	if ((!$error) && (!&File::Copy::move($filetmp,$filebak))) { $error=1; }
 	if (!$error) {
 	    &Apache::lonxml::info("<p><b>".
 				  &mt("Undid changes, Switched [_1] and [_2]",
@@ -725,11 +751,14 @@ sub analyze_header {
             <form name="lonhomework" method="POST" action="'.
 	    &HTML::Entities::encode($env{'request.uri'},'<>&"').'">'.
 	    &Apache::structuretags::remember_problem_state().'
+            <div class="LC_edit_problem_analyze_header">
             <input type="submit" name="problemmode" value="'.&mt("EditXML").'" />
             <input type="submit" name="problemmode" value="'.&mt('Edit').'" />
-            <hr />
+            <hr class="LC_edit_problem_divide" />
+
             <input type="submit" name="submit" value="'.&mt("View").'" />
-            <hr />
+            <hr class="LC_edit_problem_divide" />
+            </div>
             </form>';
     &Apache::lonxml::add_messages(\$result);
     $request->print($result);
@@ -789,12 +818,20 @@ sub analyze {
 			$i++;
 		    }
 		}
+		if (!keys(%{ $analyze{$part.'.answer'} })) {
+		    my $answer_part = 
+			['<span class="LC_error">'.&mt('Error').'</span>'];
+		    $seedexample{join("\0",$part,0,@{$answer_part})}=
+			$thisseed;
+		    push( @{ $overall{$part.'.answer'}[0] },
+			  $answer_part);
+		}
 	    }
 	}
     }
     &Apache::lonhtmlcommon::Update_PrgWin($request,\%prog_state,
 					  &mt('Analyzing Results'));
-    $request->print('<hr />'.&mt('List of possible answers').': ');
+    $request->print('<hr class="LC_edit_problem_divide" />'.&mt('List of possible answers').': ');
     foreach my $part (sort(keys(%allparts))) {
 	if (defined(@{ $overall{$part.'.answer'} })) {
 	    for (my $i=0;$i<scalar(@{ $overall{$part.'.answer'} });$i++) {
@@ -832,6 +869,41 @@ sub analyze {
     return $result;
 }
 
+{
+    my $show_problem_status;
+    sub reset_show_problem_status {
+	undef($show_problem_status);
+    }
+
+    sub set_show_problem_status {
+	my ($new_status) = @_;
+	$show_problem_status = lc($new_status);
+    }
+
+    sub hide_problem_status {
+	return ($show_problem_status eq 'no'
+		|| $show_problem_status eq 'no_feedback_ever');
+    }
+
+    sub show_problem_status {
+	return ($show_problem_status eq 'yes'
+		|| $show_problem_status eq 'answer'
+		|| $show_problem_status eq '');
+    }
+    
+    sub show_some_problem_status {
+	return ($show_problem_status eq 'no');
+    }
+
+    sub show_no_problem_status {
+	return ($show_problem_status eq 'no_feedback_ever');
+    }
+  
+    sub show_answer_problem_status {
+	return ($show_problem_status eq 'answer');
+    }
+}
+
 sub editxmlmode {
     my ($request,$file) = @_;
     my $result;
@@ -847,7 +919,7 @@ sub editxmlmode {
 	if (!$error) { $problem=&Apache::lonnet::getfile($file); }
     }
     &Apache::lonhomework::showhashsubset(\%env,'^form');
-    if ( $env{'form.submit'} eq &mt('Submit Changes and View') ) {
+    if ( $env{'form.submit'} eq &mt('Save and View') ) {
 	&Apache::lonhomework::showhashsubset(\%env,'^form');
 	$env{'form.problemmode'}='View';
 	&renderpage($request,$file);
@@ -862,30 +934,51 @@ sub editxmlmode {
 	if ($cols > 80) { $cols = 80; }
 	if ($cols < 70) { $cols = 70; }
 	if ($rows < 20) { $rows = 20; }
+	my $js =
+	    &Apache::edit::js_change_detection(). 
+	    &Apache::loncommon::resize_textarea_js();
+	my $only_body =  ($env{'environment.remote'} eq 'off')? 0 : 1;
 	my $start_page = 
-	    &Apache::loncommon::start_page(&mt("EditXML [_1]",$file),
-					   &Apache::edit::js_change_detection(),
-					   {'no_auto_mt_title' => 1,});
+	    &Apache::loncommon::start_page(&mt("EditXML [_1]",$file),$js,
+					   {'no_auto_mt_title' => 1,
+					    'only_body'        => $only_body,
+					    'add_entries'      => {
+						'onresize' => q[resize_textarea('LC_editxmltext','LC_aftertextarea')],
+						'onload'   => q[resize_textarea('LC_editxmltext','LC_aftertextarea')],
+				    }});
 
 	$result.=$start_page.
 	    &renderpage($request,$file,['no_output_web'],1).
-	    &Apache::lonxml::message_location().'
-            <form '.&Apache::edit::form_change_detection().' name="lonhomework" method="POST" action="'.
+            '<form '.&Apache::edit::form_change_detection().' name="lonhomework" method="POST" action="'.
 	    &HTML::Entities::encode($env{'request.uri'},'<>&"').'">'.
 	    &Apache::structuretags::remember_problem_state().'
-            <input type="hidden" name="problemmode" value="'.&mt('EditXML').'" />
-            <input type="submit" name="problemmode" accesskey="d" value="'.&mt('Discard Edits and View').'" />
-            <input type="submit" '.&Apache::edit::submit_ask_anyway().'name="problemmode" accesskey="e" value="'.&mt('Edit').'" />
-            <input type="submit" name="Undo" accesskey="u" value="'.&mt('undo').'" />
-            <hr />
-            <input type="submit" name="submit" accesskey="s" value="'.&mt('Submit Changes').'" />
-            <input type="submit" name="submit" accesskey="v" value="'.&mt('Submit Changes and View').'" />
-            <hr />
-            ' . $xml_help . '
-            <textarea '.&Apache::edit::element_change_detection().' style="width:100%" rows="'.$rows.'" cols="'.$cols.'" name="editxmltext">'.
-	    &HTML::Entities::encode($problem,'<>&"').'</textarea><br />
-            <input type="submit" name="submit" accesskey="s" value="'.&mt('Submit Changes').'" />
-            <input type="submit" name="submit" accesskey="v" value="'.&mt('Submit Changes and View').'" />
+            <div class="LC_edit_problem_editxml_header">
+              <table class="LC_edit_problem_header_title"><tr><td>
+               '.&mt('Problem Editing').&Apache::loncommon::help_open_topic('Problem_Editor_XML_Index').'
+                </td><td align="right">
+                  '.&Apache::loncommon::helpLatexCheatsheet().'
+                </td></tr>
+              </table>
+              <div class="LC_edit_problem_discards">
+                <input type="hidden" name="problemmode" value="'.&mt('EditXML').'" />
+                <input type="submit" name="problemmode" accesskey="d" value="'.&mt('Discard Edits and View').'" />
+                <input type="submit" '.&Apache::edit::submit_ask_anyway().'name="problemmode" accesskey="e" value="'.&mt('Edit').'" />
+                <input type="submit" name="Undo" accesskey="u" value="'.&mt('undo').'" />
+              </div>
+              <div class="LC_edit_problem_saves">
+                <input type="submit" name="submit" accesskey="s" value="'.&mt('Save').'" />
+                <input type="submit" name="submit" accesskey="v" value="'.&mt('Save and View').'" />
+              </div>
+              <hr class="LC_edit_problem_divide" />
+	      '.&Apache::lonxml::message_location().'
+            </div>
+            '  . '
+            <textarea '.&Apache::edit::element_change_detection().
+	              ' rows="'.$rows.'" cols="'.$cols.'" style="width:100%" '.
+		      ' name="editxmltext" id="LC_editxmltext">'.
+		      &HTML::Entities::encode($problem,'<>&"').'</textarea>
+            <div id="LC_aftertextarea">
+            </div>
             </form>'.&Apache::loncommon::end_page();
 	&Apache::lonxml::add_messages(\$result);
 	$request->print($result);
@@ -976,52 +1069,49 @@ sub finished_parsing {
     undef($Apache::lonhomework::parsing_a_task);
 }
 
-# with no arg it returns a HTML <option> list of the template titles
-# with one arg it returns the filename associated with the arg passed
 sub get_template_list {
-    my ($namewanted,$extension) = @_;
+    my ($extension) = @_;
     my $result;
     my @allnames;
     &Apache::lonxml::debug("Looking for :$extension:");
-    foreach my $file (</home/httpd/html/res/adm/includes/templates/*.$extension>) {
-	&Apache::lonxml::debug("Looking at $file");
-	my $name=&Apache::lonnet::metadata($file,'title');
-	&Apache::lonxml::debug("Got a name $name");
-	if ($namewanted && ($name eq $namewanted)) {
-	    $result=$file;
-	    last;
-	} else {
-	    if ($name) { push (@allnames, $name); }
-	}
+    my $glob_extension  = $extension;
+    if ($extension eq 'survey' || $extension eq 'exam') {
+	$glob_extension = 'problem';
     }
-    if (@allnames && !$result) {
-	$result="<option>".&mt("Select a")." $extension ".&mt('template')."</option>\n<option>".
-	    join('</option><option>',sort(@allnames)).'</option>';
+    my @files = glob($Apache::lonnet::perlvar{'lonIncludes'}.
+		     '/templates/*.'.$glob_extension);
+    @files = map {[$_,&mt(&Apache::lonnet::metadata($_, 'title'))]} (@files);
+    @files = sort {$a->[1] cmp $b->[1]} (@files);
+    foreach my $file (@files) {
+	next if ($file->[1] !~ /\S/);
+	$result .=
+	    '<label><input type="radio" name="template" value="'.$file->[0].'" />'.
+	    $file->[1].'</label><br />';
     }
     return $result;
 }
 
 sub newproblem {
     my ($request) = @_;
-    my $extension=$request->uri;
-    $extension=~s:^.*\.([\w]+)$:$1:;
-    &Apache::lonxml::debug("Looking for :$extension:");
-    my $templatelist=&get_template_list('',$extension);
-    if ($env{'form.template'} &&
-	$env{'form.template'} ne "Select a $extension template") {
-	use File::Copy;
-	my $file = &get_template_list($env{'form.template'},$extension);
+
+    if ($env{'form.template'}) {
+	my $file = $env{'form.template'};
 	my $dest = &Apache::lonnet::filelocation("",$request->uri);
-	copy($file,$dest);
+	&File::Copy::copy($file,$dest);
 	&renderpage($request,$dest);
-    } elsif($env{'form.newfile'} && !$templatelist) {
-	# I don't like hard-coded filenames but for now, this will work.
-	use File::Copy;
+	return;
+    }
+
+    my ($extension) = ($request->uri =~ m/\.(\w+)$/);
+    &Apache::lonxml::debug("Looking for :$extension:");
+    my $templatelist=&get_template_list($extension);
+    if ($env{'form.newfile'} && !$templatelist) {
+	# no templates found
 	my $templatefilename =
 	    $request->dir_config('lonIncludes').'/templates/blank.'.$extension;
 	&Apache::lonxml::debug("$templatefilename");
 	my $dest = &Apache::lonnet::filelocation("",$request->uri);
-	copy($templatefilename,$dest);
+	&File::Copy::copy($templatefilename,$dest);
 	&renderpage($request,$dest);
     } else {
 	my $url=&HTML::Entities::encode($request->uri,'<>&"');
@@ -1029,9 +1119,6 @@ sub newproblem {
 	$shownurl=~s-^/~-/priv/-;
 	my $dest = &Apache::lonnet::filelocation("",$request->uri);
 	my $errormsg;
-	if ($env{'form.newfile'}) {
-	    $errormsg='<p><span class="LC_error">'.&mt('You did not select a template.').'</span></p>'."\n";
-	}
 	my $instructions;
 	my $start_page = 
 	    &Apache::loncommon::start_page("Create New $extension");
@@ -1044,16 +1131,17 @@ $errormsg
 <p>
 <b>
 ".&mt("To create a new $extension, select a template from the".
-      "pull-down menu below. Then click on the \"Create $extension\" button.")."</b>
+      " list below. Then click on the \"Create $extension\" button.")."</b>
 </p><form action=\"$url\" method=\"POST\">");
 
 	if (defined($templatelist)) {
-	    $request->print("<select name=\"template\">$templatelist</select>");
+	    $request->print($templatelist);
 	}
-	$request->print("<br /><input type=\"submit\" name=\"newfile\" value=\"".&mt('Create')." $extension\" />");
+	$request->print('<br /><input type="submit" name="newfile" value="'.
+			&mt("Create $extension").'" />');
 	$request->print("</form></p>".&Apache::loncommon::end_page());
     }
-    return '';
+    return;
 }
 
 sub update_construct_style {
