@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # various response type definitons response definition
 #
-# $Id: response.pm,v 1.176 2007/08/10 22:04:10 albertel Exp $
+# $Id: response.pm,v 1.185 2007/11/13 22:21:46 albertel Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -451,7 +451,11 @@ sub end_customresponse {
     }
     if ($target eq 'grade' || $target eq 'web' || $target eq 'answer' || 
 	$target eq 'tex' || $target eq 'analyze') {
-	&Apache::lonxml::increment_counter(&Apache::response::repetition());
+	&Apache::lonxml::increment_counter(&Apache::response::repetition(),
+					   "$part.$id");
+	if ($target eq 'analyze') {
+	    &Apache::lonhomework::set_bubble_lines();
+	}
     }
     pop(@Apache::lonxml::namespace);
     pop(@Apache::response::custom_answer);
@@ -821,7 +825,10 @@ sub answer_footer {
 	$result .= join(' & ',@answer_bits);
 	$result .= ' \\\\ \\hline \end{tabular} \vskip 0 mm ';
     } else {
-	$result = '</tr></table>';
+	if (!$need_row_start) {
+	    $result .= '</tr>';
+	}
+	$result .= '</table>';
     }
     return $result;
 }
@@ -880,7 +887,6 @@ sub getresponse {
     if (!defined($lines)) {
 	$lines = 1;
     }
-
     my %let_to_num=('A'=>0,'B'=>1,'C'=>2,'D'=>3,'E'=>4,'F'=>5,'G'=>6,'H'=>7,
 		    'I'=>8,'J'=>9,'K'=>10,'L'=>11,'M'=>12,'N'=>13,'O'=>14,
 		    'P'=>15,'Q'=>16,'R'=>17,'S'=>18,'T'=>19,'U'=>20,'V'=>21,
@@ -891,14 +897,17 @@ sub getresponse {
 	
 	my $line;
 	for ($line = 0; $line < $lines; $line++) {
-	    $response = $env{'scantron.'.
-				 ($Apache::lonxml::counter+$offset-1+$line).
-				 '.answer'};
-	    if ($response ne "") {
+	    my $theline = $Apache::lonxml::counter+$offset-1+$line;
+	    $response = $env{"scantron.$theline.answer"};
+	    if ((defined($response)) && ($response ne "") && ($response ne " ")) {
 		last;
 	    }
 	    
 	}
+
+	# save bubbled letter for later
+	$Apache::lonhomework::results{"resource.$part.$id.scantron"}.=
+	    $response;
 	if ($resulttype ne 'letter') {
 	    if ($resulttype eq 'A is 1') {
 		$response = $let_to_num{$response}+1;
@@ -913,9 +922,6 @@ sub getresponse {
 		$response = chr(ord($response) + $line * $bubbles_per_line);
 	    }
 	}
-	# save submitted response for later
-	$Apache::lonhomework::results{"resource.$part.$id.scantron"}.=
-	    $response;
 
     } else {
 	$response = $env{$formparm};
@@ -1044,7 +1050,7 @@ sub show_answer {
     my $award  = $Apache::lonhomework::history{"resource.$part.solved"};
     my $status = $Apache::inputtags::status[-1];
     return  ( ($award =~ /^correct/
-	       && lc($Apache::lonhomework::problemstatus) ne 'no')
+	       && &Apache::lonhomework::show_problem_status())
 	      || $status eq "SHOW_ANSWER");
 }
 
@@ -1206,6 +1212,21 @@ sub check_status {
     return 2;
 }
 
+=pod
+
+=item setup_prior_tries_hash($func,$data)
+
+  Foreach each past .submission $func is called with 3 arguments
+     - the mode to set things up for (currently always 'grade')
+     - the stored .submission string
+     - The expansion of $data
+
+  $data is an array ref containing elements that are either
+    - scalars that are other elements of the history hash to pass to $func
+    - ref to data to be passed untouched to $func
+
+=cut
+
 sub setup_prior_tries_hash {
     my ($func,$data) = @_;
     my $part = $Apache::inputtags::part;
