@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA 
 # definition of tags that give a structure to a document
 #
-# $Id: structuretags.pm,v 1.392 2007/08/31 19:31:16 albertel Exp $
+# $Id: structuretags.pm,v 1.414 2007/11/20 17:53:22 albertel Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -42,7 +42,7 @@ use lib '/home/httpd/lib/perl/';
 use LONCAPA;
  
 BEGIN {
-    &Apache::lonxml::register('Apache::structuretags',('block','languageblock','instructorcomment','while','randomlist','problem','library','web','tex','part','preduedate','postanswerdate','solved','notsolved','problemtype','startouttext','endouttext','simpleeditbutton','definetag'));
+    &Apache::lonxml::register('Apache::structuretags',('block','languageblock','translated','instructorcomment','while','randomlist','problem','library','web','tex','part','preduedate','postanswerdate','solved','notsolved','problemtype','startouttext','endouttext','simpleeditbutton','definetag'));
 }
 
 sub start_web {
@@ -92,6 +92,31 @@ sub end_tex {
     return '';
 }
 
+sub homework_js {
+    return &Apache::loncommon::resize_textarea_js().
+	<<'JS';
+<script type="text/javascript">
+function setSubmittedPart (part) {
+   this.document.lonhomework.submitted.value="part_"+part;
+}
+
+function image_response_click (which, e) {
+    init_geometry();
+    if (!e) { e = window.event; } //IE
+    var input_element = document.lonhomework.elements[which];
+    var token_element = document.lonhomework.elements[which+'_token'];
+    var token = token_element.value;
+    var img_element   = document.getElementById(which+'_imageresponse');
+    var x= e.clientX-getX(img_element)+Geometry.getHorizontalScroll();
+    var y= e.clientY-getY(img_element)+Geometry.getVerticalScroll();
+    var click = x+':'+y;
+    input_element.value = click;
+    img_element.src = '/adm/randomlabel.png?token='+token+'&clickdata='+click;
+}
+</script>
+JS
+}
+
 sub page_start {
     my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$name,
 	$extra_head)=@_;
@@ -108,13 +133,7 @@ sub page_start {
 					   $parstack,$parser,$safeeval);
     }
 
-    $extra_head.= '
-<script type="text/javascript">
-function setSubmittedPart (part) {
-   this.document.lonhomework.submitted.value="part_"+part;
-}
-</script>
-';
+    $extra_head .= &homework_js();
 
     my %body_args;
     if (defined($found{'html'})) {
@@ -139,7 +158,9 @@ function setSubmittedPart (part) {
     } elsif (!defined($found{'body'}) 
 	     && $env{'request.state'} eq 'construct') {
 	if ($target eq 'web' || $target eq 'edit') {
-	    # no extra args to bodytag
+	    if ($env{'environment.remote'} ne 'off') {
+		$body_args{'only_body'}  = 1;
+	    }
 	}
     } elsif (!defined($found{'body'})) {
 	my %add_entries;
@@ -178,7 +199,7 @@ function setSubmittedPart (part) {
 	}
     }
 
-    if (!defined($found{'body'})) {
+    if (!defined($found{'body'}) && $env{'request.state'} ne 'construct') {
 	$page_start .= &Apache::lonxml::message_location();
     }
     
@@ -281,20 +302,44 @@ sub remember_problem_state {
 sub problem_edit_header {
     return '<input type="hidden" name="submitted" value="edit" />'.
 	&Apache::structuretags::remember_problem_state().'
+<div class="LC_edit_problem_header">
+<div class="LC_edit_problem_header_title">
+'.&mt('Problem Editing').&Apache::loncommon::help_open_menu('Problem Editing','Problem_Editor_XML_Index',5,'Authoring').'
+</div>
+<div class="LC_edit_problem_discards">
        <input type="hidden" name="problemmode" value="'.&mt('Edit').'" />
        <input type="submit" name="problemmode" accesskey="d" value="'.&mt('Discard Edits and View').'" />
        <input '.&Apache::edit::submit_ask_anyway().' type="submit" name="problemmode" accesskey="x" value="'.&mt('EditXML').'" />
-       <input type="submit" name="Undo" accesskey="u" value="'.&mt('undo').'" /> <hr />
-       <input type="submit" name="submit" accesskey="s" value="'.&mt('Submit Changes and Edit').'" />
-       <input type="submit" name="submit" accesskey="v" value="'.&mt('Submit Changes and View').'" /><table><tr><td>'.
-       &Apache::loncommon::help_open_menu('Problem Editing Help','Problem_Editor_XML_Index',5,'Authoring',undef,undef,undef,'Problem Editing Help')
-       .'</td></tr></table>'.
+       <input type="submit" name="Undo" accesskey="u" value="'.&mt('undo').'" />
+</div>
+<div class="LC_edit_problem_saves">
+       <input type="submit" name="submit" accesskey="s" value="'.&mt('Save and Edit').'" />
+       <input type="submit" name="submit" accesskey="v" value="'.&mt('Save and View').'" />
+  </div>
+<hr class="LC_edit_problem_divide" />
+'.&Apache::lonxml::message_location().'
+</div>
+'.
        '<table border="0" width="100%"><tr><td bgcolor="#DDDDDD">';
 }
 
 sub problem_edit_footer {
-    return '</td></tr></table><br /><input type="submit" name="submit" value="'.&mt('Submit Changes and Edit').'" />
-    <input type="submit" name="submit" value="'.&mt('Submit Changes and View').'" />'.
+    return '</td></tr></table><br />
+<div class="LC_edit_problem_footer">
+  <hr class="LC_edit_problem_divide" />
+  <div class="LC_edit_problem_discards">
+     <input type="submit" name="problemmode" accesskey="d" value="'.&mt('Discard Edits and View').'" />
+     <input '.&Apache::edit::submit_ask_anyway().' type="submit" name="problemmode" accesskey="x" value="'.&mt('EditXML').'" />
+     <input type="submit" name="Undo" accesskey="u" value="'.&mt('undo').'" />
+  </div>
+  <div class="LC_edit_problem_saves">
+    <input type="submit" name="submit" value="'.&mt('Save and Edit').'" />
+    <input type="submit" name="submit" value="'.&mt('Save and View').'" />
+  </div>
+  <hr class="LC_edit_problem_divide" />
+</div>
+'.
+
     &Apache::lonhtmlcommon::htmlareaselectactive(&Apache::lonhtmlcommon::get_htmlareafields()).
     "\n</form>\n".&Apache::loncommon::end_page();
 }
@@ -311,35 +356,38 @@ sub option {
 
 sub problem_web_to_edit_header {
     my ($rndseed)=@_;
-    my $result.='<input type="hidden" name="problemmode" value="'.&mt('View').'" />';
-    $result .= '<input type="submit" name="problemmode" accesskey="e" value="'.&mt('Edit').'" />';
-    $result .= '<input type="submit" name="problemmode" accesskey="x" value="'.&mt('EditXML').'" />
-             <input type="submit" name="newrandomization" accesskey="a" value="'.&mt('New Randomization').'" />
-             <input type="submit" name="resetdata" accesskey="r" value="'.&mt('Reset Submissions').'" />
-             <nobr><input type="submit" name="changerandseed" value="'.&mt('Change Random Seed To:').'" />
-              <input type="text" name="rndseed" size="10" value="'.
-	       $rndseed.'"
-           onchange="javascript:document.lonhomework.changerandseed.click()" /></nobr>
-             <span class="LC_nobreak"><label><input type="checkbox" name="showallfoils" ';
+    my $result .= '<div class="LC_edit_problem_header">';
+
+    if (!$Apache::lonhomework::parsing_a_task) {
+	$result .= 
+	    '<div class="LC_edit_problem_header_title">'.
+	    &mt('Problem Testing').
+	    &Apache::loncommon::help_open_topic('Problem_Editor_Testing_Area').
+	    '</div>';
+    } else {
+	$result .= 
+	    '<div class="LC_edit_problem_header_title">'.
+	    &mt('Task Testing').
+	    '</div>';
+    }
+    
     my $show_all_foils_text = 
 	($Apache::lonhomework::parsing_a_task) ?
 	&mt('&nbsp;Show&nbsp;All&nbsp;Instances')
 	: &mt('&nbsp;Show&nbsp;All&nbsp;Foils');
 
-    if (defined($env{'form.showallfoils'})) { $result.='checked="on"'; }
-    $result.= ' />'.$show_all_foils_text.'</label></span>'.
-	&Apache::loncommon::help_open_topic('Problem_Editor_Testing_Area','Testing Problems').
-	'<hr />';
-    $result.=&mt('Apply style file: ').'
-<input type="text" name="style_file" value="'.&HTML::Entities::encode($env{'construct.style'},'"<>&').'" />
- <a href="javascript:openbrowser(\'lonhomework\',\'style_file\',\'sty\')">'.&mt('Select').'</a>
-<input type="submit" name="clear_style_file" value="'.&mt('Clear').'" />
-<br />';
+    my $show_all= '<span class="LC_nobreak"><label><input type="checkbox" name="showallfoils" ';
+    if (defined($env{'form.showallfoils'})) { $show_all.='checked="checked"'; }
+    $show_all.= ' />'.$show_all_foils_text.'</label></span>';
 
+
+
+    $result .= '<div class="LC_edit_problem_header_status_row">';
     if (!$Apache::lonhomework::parsing_a_task) {
 	$result.="
-<nobr>
-Problem Status:
+<div class='LC_edit_problem_header_row1'>
+<span class=\"LC_nobreak\">
+".&mt("Problem Status:")."
 <select name='problemstate'>
   <option value=''></option>
   ".&option('CLOSED'               ,'problemstate').&mt("Closed")."</option>
@@ -348,39 +396,35 @@ Problem Status:
   ".&option('CANNOT_ANSWER_correct','problemstate').&mt("Open and correct")."</option>
   ".&option('SHOW_ANSWER'          ,'problemstate').&mt("Show Answer")."</option>
 </select>
-</nobr>
-<nobr>
-Problem Type:
+</span>
+<span class=\"LC_nobreak\">
+".&mt("Problem Type:")."
 <select name='problemtype'>
   <option value=''></option>
   ".&option('exam'   ,'problemtype').&mt("Exam Problem")."</option>
   ".&option('problem','problemtype').&mt("Homework problem")."</option>
   ".&option('survey' ,'problemtype').&mt("Survey Question")."</option>
 </select>
-</nobr>
-<nobr>
-Feedback Mode:
+</span>
+$show_all
+</div>
+<div class='LC_edit_problem_header_row2'>
+<span class=\"LC_nobreak\">
+".&mt("Feedback Mode:")."
 <select name='problemstatus'>
   <option value=''></option>
   ".&option('yes','problemstatus').&mt("Show Feedback")."</option>
-  ".&option('no', 'problemstatus').&mt("Don't Show Feedback")."</option>
+  ".&option('no', 'problemstatus').&mt("Don't Show Incorect/Correct Feedback")."</option>
+  ".&option('no_feedback_ever', 'problemstatus').&mt("Don't Show Any Feedback")."</option>
 </select>
-</nobr>
-<input type='submit' name='changeproblemmode' value='".&mt("Change")."' />
-<hr />";
-	my $numtoanalyze=$env{'form.numtoanalyze'};
-	if (!$numtoanalyze) { $numtoanalyze=20; }
-	$result.= '<input type="submit" name="problemmode" value='.
-	    &mt('"Calculate answers').'" /> for
-             <input type="text" name="numtoanalyze" value="'.
-	     $numtoanalyze.'" size="5" /> '.&mt('versions of this problem').
-	     '.'.&Apache::loncommon::help_open_topic("Analyze_Problem",
-						     '',undef,undef,300).
-						     '<hr />';
+</span>
+";
+
     } elsif ($Apache::lonhomework::parsing_a_task) {
 	$result.="
-<nobr>
-Problem Status:
+<div class='LC_edit_problem_header_row1'>
+<span class=\"LC_nobreak\">
+".&mt("Problem Status:")."
 <select name='problemstate'>
   <option value=''></option>
   ".&option('CLOSED'               ,'problemstate').&mt("Closed")."</option>
@@ -388,10 +432,53 @@ Problem Status:
   ".&option('WEB_GRADE'            ,'problemstate').&mt("Criteria Grading")."</option>
   ".&option('SHOW_ANSWER'          ,'problemstate').&mt("Show Feedback")."</option>
 </select>
-</nobr>
-<input type='submit' name='changeproblemmode' value='".&mt("Change")."' />
-<hr />";
+</span>
+$show_all
+";
     }
+
+    $result.='
+       <span class="LC_nobreak">
+       '.&mt('Apply style file: ').'
+         <input type="text" name="style_file" value="'.&HTML::Entities::encode($env{'construct.style'},'"<>&').'" />
+         <a href="javascript:openbrowser(\'lonhomework\',\'style_file\',\'sty\')">'.&mt('Select').'</a>
+       </span>
+     </div>
+     <input type="submit" name="changeproblemmode" value="'.&mt("Change View").'" />
+     <input type="submit" name="clear_style_file" accesskey="d" value="'.&mt('Show Default View').'" />
+     <input type="submit" name="resetdata" accesskey="r" value="'.&mt('Reset Submissions').'" />
+   </div>
+   <hr class="LC_edit_problem_divide" />
+   <div class="LC_edit_problem_header_randomize_row">
+     <input type="submit" name="newrandomization" accesskey="a" value="'.&mt('New Randomization').'" />
+     <input type="submit" name="changerandseed" value="'.&mt('Change Random Seed To:').'" />
+     <input type="text" name="rndseed" size="10" value="'.
+	       $rndseed.'"
+             onchange="javascript:document.lonhomework.changerandseed.click()" />';
+
+    if (!$Apache::lonhomework::parsing_a_task) {
+	my $numtoanalyze=$env{'form.numtoanalyze'};
+	if (!$numtoanalyze) { $numtoanalyze=20; }
+	$result .= '<span class="LC_nobreak">'.
+	    &mt('[_1] for [_2] versions.',
+		'<input type="submit" name="problemmode" value='.&mt('"Calculate answers').'" />',
+		'<input type="text" name="numtoanalyze" value="'.
+		$numtoanalyze.'" size="5" />').
+		&Apache::loncommon::help_open_topic("Analyze_Problem",'',undef,undef,300).
+		'</span>';
+						    
+    }
+
+    $result.='
+   </div>
+   <div class="LC_edit_problem_header_edit_row">';
+    $result.='<input type="hidden" name="problemmode" value="'.&mt('View').'" />';
+    $result .= '<input type="submit" name="problemmode" accesskey="e" value="'.&mt('Edit').'" />';
+    $result .= '<input type="submit" name="problemmode" accesskey="x" value="'.&mt('EditXML').'" />';
+    $result.='
+   </div>
+   '.&Apache::lonxml::message_location().'
+</div>';
     return $result;
 }
 
@@ -533,17 +620,36 @@ ENDCHECKOUT
 
 sub firstaccess_msg {
     my ($time,$symb)=@_;
-    my ($map)=&Apache::lonnet::decode_symb($symb);
-    my $foldertitle=&Apache::lonnet::gettitle($map);
-    &Apache::lonxml::debug("map is $map title is $foldertitle");
-    my $uri = &Apache::lonenc::check_encrypt($env{'request.uri'});
-    return (<<ENDCHECKOUT);
+    my $result;
+    my @interval=&Apache::lonnet::EXT("resource.0.interval");
+    if ($interval[1] eq 'map') {
+	my ($map)=&Apache::lonnet::decode_symb($symb);
+	my $foldertitle=&Apache::lonnet::gettitle($map);
+    
+	&Apache::lonxml::debug("map is $map title is $foldertitle");
+	$result .= (<<ENDCHECKOUT);
 <h2>The resources in "$foldertitle" are open for a limited time. Once you click the 'Show Resource' button below you have $time to complete all resources "$foldertitle".</h2>
+ENDCHECKOUT
+    } elsif ($interval[1] eq 'course') {
+	my $course = $env{'course.'.$env{'request.course.id'}.'.description'};
+	$result .= (<<ENDCHECKOUT);
+<h2>The resources in "$course" are open for a limited time. Once you click the 'Show Resource' button below you have $time to complete all resources in "$course".</h2>
+ENDCHECKOUT
+    } else {
+	my $title=&Apache::lonnet::gettitle($symb);
+	$result .= (<<ENDCHECKOUT);
+<h2>This resource "$title" is open for a limited time. Once you click the 'Show Resource' button below you have $time to complete this resource "$title".</h2>
+ENDCHECKOUT
+
+    }
+    my $uri = &Apache::lonenc::check_encrypt($env{'request.uri'});
+    $result .= (<<ENDCHECKOUT);
 <form name="markaccess" method="POST" action="$uri">
 <input type="hidden" name="markaccess" value="yes" />
 <input type="button" name="accessbutton" value="Show Resource" onClick="javascript:if (confirm('Start Timer?')) { document.markaccess.submit(); }" />
 </form>
 ENDCHECKOUT
+    return $result;
 }
 
 sub init_problem_globals {
@@ -555,19 +661,19 @@ sub init_problem_globals {
     if ($type eq 'problem') {
 	$Apache::inputtags::part='0';
 	@Apache::inputtags::partlist=('0');
-	$Apache::lonhomework::problemstatus=&get_problem_status('0');
+	&Apache::lonhomework::set_show_problem_status(&get_problem_status('0'));
 	$Apache::lonhomework::ignore_response_errors=0;
 
     } elsif ($type eq 'library') {
 	$Apache::inputtags::part='';
 	@Apache::inputtags::partlist=();
-	$Apache::lonhomework::problemstatus='';	
+	&Apache::lonhomework::reset_show_problem_status();
 	$Apache::lonhomework::ignore_response_errors=1;
 
     } elsif ($type eq 'Task') {
 	$Apache::inputtags::part='0';
 	@Apache::inputtags::partlist=('0');
-	$Apache::lonhomework::problemstatus='';	
+	&Apache::lonhomework::reset_show_problem_status();
 	$Apache::lonhomework::ignore_response_errors=1;
     }
     @Apache::inputtags::responselist = ();
@@ -596,8 +702,8 @@ sub reset_problem_globals {
     undef($Apache::lonhomework::default_type);
     undef($Apache::lonhomework::type);
     undef($Apache::lonhomework::scantronmode);
-    undef($Apache::lonhomework::problemstatus);
     undef($Apache::lonhomework::ignore_response_errors);
+    &Apache::lonhomework::reset_show_problem_status();
 }
 
 sub set_problem_state {
@@ -650,7 +756,7 @@ sub start_problem {
     if ($target ne 'analyze') {
 	$Apache::lonhomework::type=&Apache::lonnet::EXT('resource.0.type');
 	if (($env{'request.state'} eq 'construct') &&
-	    defined($env{'form.problemtype'})) {
+	    $env{'form.problemtype'} =~ /\S/) {
 	    $Apache::lonhomework::type=$env{'form.problemtype'};
 	}
 	&Apache::lonxml::debug("Found this to be of type :$Apache::lonhomework::type:");
@@ -711,7 +817,8 @@ sub start_problem {
 	    $result.=&Apache::lonxml::printtokenheader($target,$token);
 	}
 	if ($env{'form.markaccess'}) {
-	    &Apache::lonnet::set_first_access('map');
+	    my @interval=&Apache::lonnet::EXT("resource.0.interval");
+	    &Apache::lonnet::set_first_access($interval[1]);
 	}
 	#handle rand seed in construction space
 	my $rndseed=&setup_rndseed($safeeval);
@@ -765,7 +872,7 @@ sub start_problem {
 		    $msg.='<h1>'.&mt('Not open to be viewed').'</h1>';
 		}
 		if ($status eq 'CLOSED' || $status eq 'INVALID_ACCESS') {
-		    $msg.='The problem '.$accessmsg;
+		    $msg.=&mt('The problem ').$accessmsg;
 		} elsif ($status eq 'UNCHECKEDOUT') {
 		    $msg.=&checkout_msg();
 		} elsif ($status eq 'NOT_YET_VIEWED') {
@@ -1070,16 +1177,19 @@ sub start_definetag {
 
     my $name = $token->[2]->{'name'};
     my $skip=&Apache::lonxml::get_all_text("/definetag",$parser,$style);
-    if ($name=~/^\//) {
-	$result=
- '<br /><table bgcolor="#FFBBBB"><tr><th>END <tt>'.$name.'</tt></th></tr>';
-    } else {
-	$result=
- '<br /><table bgcolor="#BBFFBB"><tr><th>BEGIN <tt>'.$name.'</tt></th></tr>';
+    if ($target eq 'web') {
+	if ($name=~/^\//) {
+	    $result=
+		'<br /><table class="LC_sty_end"><tr><th>'.
+		&mt('END [_1]'.'<tt>'.$name.'</tt>').'</th></tr>';
+	} else {
+	    $result=
+		'<br /><table class="LC_sty_begin"><tr><th>'.
+		&mt('BEGIN [_1]'.'<tt>'.$name.'</tt>').'</th></tr>';
+	}
+	$skip = &HTML::Entities::encode($skip, '<>&"');
+	$result.='<tr><td><pre>'.$skip.'</pre></td></tr></table>';
     }
-    $skip=~s/\</\&lt\;/gs;
-    $skip=~s/\>/\&gt\;/gs;
-    $result.='<tr><td><pre>'.$skip.'</pre></td></tr></table>';
     return $result;
 }
 
@@ -1140,19 +1250,38 @@ sub start_languageblock {
 	$target eq 'tex' || $target eq 'analyze' || $target eq 'webgrade') {
 	my $include = $token->[2]->{'include'};
 	my $exclude = $token->[2]->{'exclude'};
-        my %languages=&Apache::loncommon::display_languages();
-        $result='1';
-	if ($include) {
-            $result='';
-            foreach (split(/\,/,$include)) {
-                if ($languages{$_}) { $result='1'; }
-            }
-	}
-        if ($exclude) {
-            foreach (split(/\,/,$exclude)) {
-                if ($languages{$_}) { $result='0'; }
-            }
-	}
+        my @preferred_languages=&Apache::loncommon::preferred_languages();
+# This should not even happen, since we should at least have the server language
+        if (!$preferred_languages[0]) { $preferred_languages[0]='en'; }
+# Now loop over all languages in order of preference
+        foreach my $preferred_language (@preferred_languages) {
+# If the languageblock has no arguments, show the contents
+           $result=1;
+           my $found=0;
+# Do we have an include argument?
+	   if ($include) {
+# If include is specified, by default, don't render the block
+              $result=0;
+              foreach my $included_language (split(/\,/,$include)) {
+# ... but if my preferred language is included, render it
+                 if ($included_language eq $preferred_language) {
+                    $result=1; 
+                    $found=1; 
+                 }
+              }
+	   }
+# Do we have an exclude argument?
+           if ($exclude) {
+              $result=1;
+              foreach my $excluded_language (split(/\,/,$exclude)) {
+                 if ($excluded_language eq $preferred_language) {
+                    $result=0;
+                    $found=1;
+                 }
+              }
+	   }
+           if ($found) { last; }
+        }
 	if ( ! $result ) {
 	    my $skip=&Apache::lonxml::get_all_text("/languageblock",$parser,
 						   $style);
@@ -1184,6 +1313,56 @@ sub end_languageblock {
     return $result;
 }
 
+{
+    my %available_texts;
+    sub start_translated {
+	my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
+	&Apache::lonxml::register('Apache::structuretags',('lang'));
+	undef(%available_texts);
+    }
+    
+    sub end_translated {
+	my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
+	my $result;
+	#show the translation on viewable targets
+	if ($target eq 'web'     || $target eq 'tex' || $target eq 'webgrade'||
+	    # or non-viewable targets, if it's embedded in something that
+	    # wants the output
+	    (($target eq 'answer' || $target eq 'analyze'|| $target eq 'grade')
+	     && &Apache::lonxml::in_redirection() ) ) {
+	    my @possibilities = keys(%available_texts);
+	    my $which = 
+		&Apache::loncommon::languages(\@possibilities) || 'default';
+	    $result = $available_texts{$which};
+	}
+	undef(%available_texts);
+	&Apache::lonxml::deregister('Apache::structuretags',('lang'));
+	return $result;
+    }
+
+
+    sub start_lang {
+	my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
+	if ($target eq 'web' || $target eq 'grade'   || $target eq 'answer' ||
+	    $target eq 'tex' || $target eq 'analyze' || $target eq 'webgrade') {
+	    &Apache::lonxml::startredirection();
+	}
+	return '';
+    }
+
+    sub end_lang {
+	my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
+	if ($target eq 'web' || $target eq 'grade'   || $target eq 'answer' ||
+	    $target eq 'tex' || $target eq 'analyze' || $target eq 'webgrade') {
+	    my $result = &Apache::lonxml::endredirection();
+	    my $which = &Apache::lonxml::get_param('which',$parstack,
+						   $safeeval);
+	    $available_texts{$which} = $result;
+	}
+	return '';
+    }
+}
+
 sub start_instructorcomment {
     my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
 
@@ -1395,7 +1574,9 @@ sub start_part {
     @Apache::inputtags::response=();
     @Apache::inputtags::previous=();
     @Apache::inputtags::previous_version=();
-    $Apache::lonhomework::problemstatus=&get_problem_status($id);
+    &Apache::lonhomework::set_show_problem_status(&get_problem_status($id));
+    &Apache::response::reset_params();
+
     my $hidden=&Apache::loncommon::check_if_partid_hidden($Apache::inputtags::part);
     my $newtype=&Apache::lonnet::EXT("resource.$id.type");
     if ($newtype) { $Apache::lonhomework::type=$newtype; }
@@ -1652,6 +1833,13 @@ sub start_startouttext {
     my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
     my @result=(''.'');
     if ($target eq 'edit' || $target eq 'modified' ) { @result=('','no'); }
+    
+    my $nesting = 
+	&Apache::lonxml::set_state('outtext',
+				   &Apache::lonxml::get_state('outtext')+1);
+    if ($nesting > 1 && $env{'request.state'} eq 'construct') {
+	&Apache::lonxml::error("Nesting of &lt;startouttext /&gt; not allowed, on line ".$token->[5]);
+    }
     return (@result);
 }
 
@@ -1691,6 +1879,14 @@ sub start_endouttext {
 	$result='<endouttext />'.
 	    &Apache::edit::handle_insertafter('startouttext');
     }
+
+    my $nesting = 
+	&Apache::lonxml::set_state('outtext',
+				   &Apache::lonxml::get_state('outtext')-1);
+    if ($nesting < 0 && $env{'request.state'} eq 'construct') {
+	&Apache::lonxml::error(" Extraneous &lt;endouttext /&gt; not allowed on line ".$token->[5]);
+	&Apache::lonxml::set_state('outtext', 0);
+    }
     return $result;
 }
 
