@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # a pile of common html routines
 #
-# $Id: lonhtmlcommon.pm,v 1.161 2007/09/05 05:00:24 raeburn Exp $
+# $Id: lonhtmlcommon.pm,v 1.171 2007/11/17 01:46:03 albertel Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -481,6 +481,19 @@ ENDJS
     return $result;
 }
 
+
+sub build_url {
+    my ($base, $fields)=@_;
+    my $url;
+    $url = $base.'?';
+    foreach my $key (keys(%$fields)) {
+        $url.=&escape($key).'='.&escape($$fields{$key}).'&amp;';
+    }
+    $url =~ s/&amp;$//;
+    return $url;
+}
+
+
 ##############################################
 ##############################################
 
@@ -632,6 +645,8 @@ sub javascript_nothing {
 ##############################################
 ##############################################
 sub javascript_docopen {
+    my ($mimetype) = @_;
+    $mimetype ||= 'text/html';
     # safari does not understand document.open() and loads "text/html"
     my $nothing = "''";
     my $user_browser;
@@ -645,7 +660,7 @@ sub javascript_docopen {
     if ($user_browser eq 'safari' && $user_os =~ 'mac') {
         $nothing = "document.clear()";
     } else {
-	$nothing = "document.open('text/html','replace')";
+	$nothing = "document.open('$mimetype','replace')";
     }
     return $nothing;
 }
@@ -682,7 +697,7 @@ Returns: a perl string as described.
 ##############################################
 ##############################################
 sub StatusOptions {
-    my ($status, $formName,$size,$onchange)=@_;
+    my ($status, $formName,$size,$onchange,$mult)=@_;
     $size = 1 if (!defined($size));
     if (! defined($status)) {
         $status = 'Active';
@@ -691,6 +706,9 @@ sub StatusOptions {
 
     my $Str = '';
     $Str .= '<select name="Status"';
+    if (defined($mult)){
+        $Str .= ' multiple="multiple" ';
+    }
     if(defined($formName) && $formName ne '' && ! defined($onchange)) {
         $Str .= ' onchange="document.'.$formName.'.submit()"';
     }
@@ -852,7 +870,7 @@ sub Create_PrgWin {
          function openpopwin () {
          popwin=open(\'\',\'popwin\',\'width=400,height=100\');".
         "popwin.document.writeln(\'".$start_page.
-              "<h4>$heading<\/h4>".
+              "<h4>".&mt("$heading")."<\/h4>".
               "<form action= \"\" name=\"popremain\" method=\"post\">".
               '<input type="text" size="'.$width.'" name="remaining" value="'.
 	      &mt('Starting').'" /><\\/form>'.$end_page.
@@ -871,8 +889,7 @@ sub Create_PrgWin {
 	}
 	if (!$inputname) {
 	    $prog_state{'inputname'}=&get_uniq_name();
-	    &r_print($r,$heading.' <input type="text" name="'.$prog_state{'inputname'}.
-		     '" size="'.$width.'" />');
+	    &r_print($r,&mt("$heading [_1]",' <input type="text" name="'.$prog_state{'inputname'}.'" size="'.$width.'" />'));
 	} else {
 	    $prog_state{'inputname'}=$inputname;
 	    
@@ -1006,7 +1023,12 @@ sub crumbs {
 	    }	    
             my $href_path = &HTML::Entities::encode($path,'<>&"');
 	    &Apache::loncommon::inhibit_menu_check(\$href_path);
-	    $output.=qq{<a href="$href_path" $target>$dir</a>/};
+	    if ($form) {
+	        my $href = 'javascript:'.$form.".action='".$href_path."';".$form.'.submit();';
+	        $output.=qq{<a href="$href" $target>$dir</a>/};
+	    } else {
+	        $output.=qq{<a href="$href_path" $target>$dir</a>/};
+	    }
 	}
     } else {
 	foreach my $dir (split('/',$uri)) {
@@ -1080,46 +1102,13 @@ ENDLINK
 }
 
 sub htmlareaheaders {
-    if (&htmlareablocked()) { return ''; }
-    unless (&htmlareabrowser()) { return ''; }
-    my $lang='en';
-    if (&mt('htmlarea_lang') ne 'htmlarea_lang') {
-	$lang=&mt('htmlarea_lang');
-    }
+    return if (&htmlareablocked());
+    return if (!&htmlareabrowser());
     return (<<ENDHEADERS);
-<script type="text/javascript">
-_editor_url='/htmlarea/';
-_editor_lang='$lang';
-</script>
-<script type="text/javascript" src="/htmlarea/htmlarea.js"></script>
-<link rel="stylesheet" type="text/css" href="/htmlarea/htmlarea.css" />
+<script type="text/javascript" src="/fckeditor/fckeditor.js"></script>
 ENDHEADERS
 }
 
-# ------------------------------------------------- Activate additional buttons
-
-sub htmlareaaddbuttons {
-    if (&htmlareablocked()) { return ''; }
-    unless (&htmlareabrowser()) { return ''; }
-    return (<<ENDADDBUTTON);
-    var config=new HTMLArea.Config();
-    config.registerButton('ed_math','LaTeX Inline',
-			  '/htmlarea/images/ed_math.gif',false,
-			    function(editor,id) {
-			      editor.surroundHTML('&nbsp;<m>\$','\$</m>&nbsp;');
-			    }
-			  );
-    config.registerButton('ed_math_eqn','LaTeX Equation',
-			  '/htmlarea/images/ed_math_eqn.gif',false,
-			    function(editor,id) {
-			      editor.surroundHTML(
-				     '&nbsp;\\n<center><m>\\\\[','\\\\]</m></center>\\n&nbsp;');
-			    }
-			  );
-    config.toolbar.push(['ed_math','ed_math_eqn']);
-ENDADDBUTTON
-}
-
 # ----------------------------------------------------------------- Preferences
 
 sub disablelink {
@@ -1138,16 +1127,33 @@ sub enablelink {
     return '<a href="'.&HTML::Entities::encode('/adm/preferences?action=set_wysiwyg&wysiwyg=on&returnurl=','<>&"').&escape($ENV{'REQUEST_URI'}).'">'.&mt('Enable WYSIWYG Editor').'</a>';
 }
 
+# ------------------------------------------------- lang to use in html editor
+sub htmlarea_lang {
+    my $lang='en';
+    if (&mt('htmlarea_lang') ne 'htmlarea_lang') {
+	$lang=&mt('htmlarea_lang');
+    }
+    return $lang;
+}
+
 # ----------------------------------------- Script to activate only some fields
 
 sub htmlareaselectactive {
     my @fields=@_;
     unless (&htmlareabrowser()) { return ''; }
     if (&htmlareablocked()) { return '<br />'.&enablelink(@fields); }
-    my $output='<script type="text/javascript" defer="1">'.
-	&htmlareaaddbuttons();
-    foreach(@fields) {
-	$output.="\nHTMLArea.replace('$_',config);";
+    my $output='<script type="text/javascript" defer="1">';
+    my $lang = &htmlarea_lang();
+    foreach my $field (@fields) {
+	$output.="
+{
+    var oFCKeditor = new FCKeditor('$field');
+    oFCKeditor.Config['CustomConfigurationsPath'] = 
+	'/fckeditor/loncapaconfig.js';    
+    oFCKeditor.ReplaceTextarea();
+    oFCKeditor.Config['AutoDetectLanguage'] = false;
+    oFCKeditor.Config['DefaultLanguage'] = '$lang';
+}";
     }
     $output.="\nwindow.status='Activated Editfields';\n</script><br />".
 	&disablelink(@fields);
@@ -1468,7 +1474,14 @@ sub course_select_row {
     my ($title,$formname,$totcodes,$codetitles,$idlist,$idlist_titles,
 	$css_class) = @_;
     my $output = &row_title($title,$css_class);
-    $output .= qq|
+    $output .= &course_selection($formname,$totcodes,$codetitles,$idlist,$idlist_titles);
+    $output .= &row_closure();
+    return $output;
+}
+
+sub course_selection {
+    my ($formname,$totcodes,$codetitles,$idlist,$idlist_titles) = @_;
+    my $output = qq|
 <script type="text/javascript">
     function coursePick (formname) {
         for  (var i=0; i<formname.coursepick.length; i++) {
@@ -1541,8 +1554,7 @@ sub course_select_row {
             $output .= '</tr></table><br />';
         }
     }
-    $output .= '<input type="radio" name="coursepick" value="specific" onclick="coursePick(this.form);opencrsbrowser('."'".$formname."'".','."'".'dccourse'."'".','."'".'dcdomain'."'".','."'".'coursedesc'."','','1'".')" />'.&mt('Pick specific course(s):').' '.$courseform.'&nbsp;&nbsp;<input type="text" value="0" size="4" name="coursetotal" /><input type="hidden" name="courselist" value="" />selected.<br />'."\n";
-    $output .= &row_closure();
+    $output .= '<input type="radio" name="coursepick" value="specific" onclick="coursePick(this.form);opencrsbrowser('."'".$formname."','dccourse','dcdomain','coursedesc','','1'".')" />'.&mt('Pick specific course(s):').' '.$courseform.'&nbsp;&nbsp;<input type="text" value="0" size="4" name="coursetotal" /><input type="hidden" name="courselist" value="" />selected.<br />'."\n";
     return $output;
 }
 
