@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Directory Indexer
 #
-# $Id: lonindexer.pm,v 1.173 2007/07/18 13:39:31 banghart Exp $
+# $Id: lonindexer.pm,v 1.179 2007/08/17 22:54:36 banghart Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -106,7 +106,7 @@ sub handler {
 
     my $fileclr='#ffffe6';
     my $line;
-    my (@attrchk,@openpath);
+    my (@attrchk,@openpath,$typeselect);
     my $uri=$r->uri;
 
 # -------------------------------------- see if called from an interactive mode
@@ -135,6 +135,11 @@ sub handler {
 	    untie(%dbfile);
 	}
     }
+# - Evaluate actions from previous page (both cumulatively and chronologically)
+        if ($env{'form.catalogmode'} eq 'import' || $hash{'form.catalogmode'} eq 'import') {
+	    &Apache::groupsort::update_actions_hash(\%hash);
+	}
+    
     {
   #Hijack lonindexer to verify a title and be close down.
    if ($env{'form.launch'} eq '2') {
@@ -354,7 +359,10 @@ END
         
 	my $js = <<"ENDHEADER";
 <script type="text/javascript">
-$catalogmodefunctions
+$catalogmodefunctions;
+function update_only(field) {
+    alert(field.name);
+}
 function checkAll() {
     var numForms = document.forms.length;
     for (i=0;i<numForms;i++) {
@@ -401,11 +409,13 @@ ENDHEADER
         my ($headerdom)=($uri=~m{^/res/($match_domain)/});
 	$r->print(&Apache::loncommon::start_page('Browse Resources',$js,
 						 {'domain' => $headerdom,}));
-# - Evaluate actions from previous page (both cumulatively and chronologically)
-        if ($env{'form.catalogmode'} eq 'import') {
-	    &Apache::groupsort::update_actions_hash(\%hash);
+# ---------------------------------- get state of file types to be showing
+	if ($env{'form.only'}) {
+	    $typeselect = $env{'form.only'};
+	} else {
+	    $typeselect = '';
 	}
-	
+
 # ---------------------------------- get state of file attributes to be showing
 	if ($env{'form.attrs'}) {
 	    for (my $i=0; $i<=11; $i++) {
@@ -437,7 +447,10 @@ ENDHEADER
 					   'sr' => 'Show resource',
 					   'li' => 'Linked/Related Resources',
 					   'av' => 'All versions',
-					   'ud' => 'Update Display'
+					   'ud' => 'Update Display',
+					   'pr' => 'Problems',
+					   'gr' => 'Graphics',
+					   'at' => 'All types'
 					   );
 	$r->print(<<END);
 <form method="post" name="fileattr" action="$uri"
@@ -464,11 +477,31 @@ ENDHEADER
 </tr>
 </table>
 <input type="hidden" name="attrs" value="1" />
+END
+        my @file_categories = &Apache::loncommon::filecategories();
+        my %select_file_categories;
+        my @select_form_order = ('');
+        $select_file_categories{''} = &mt('All file types');
+        foreach my $cat (@file_categories) {
+	    my $types = join(",",&Apache::loncommon::filecategorytypes($cat));
+	    $select_file_categories{$types} = $cat;
+	    push(@select_form_order,$types);
+	}
+        $select_file_categories{'select_form_order'} = \@select_form_order;
+        my $type_element=
+	    &Apache::loncommon::select_form($typeselect,'only',
+					    %select_file_categories);
+        $type_element = '<label>'.&mt('File Type Displayed: [_1]',
+				      $type_element).'</label>';
+        $r->print(<<END);
+$type_element
+<br />
 <input type="submit" name="updatedisplay" value="$lt{'ud'}" />
 <input type="hidden" name="acts" value="" />
 $closebutton $groupimportbutton
 END
         $r->print(&Apache::loncommon::inhibit_menu_check('input'));
+   
 # -------------- Filter out sequence containment in crumbs and "recent folders"
 	my $storeuri=$uri;
 	$storeuri='/'.(split(/\.(page|sequence)\/\//,$uri))[-1];
@@ -902,11 +935,11 @@ sub display_line {
 	my @file_ext = split (/\./,$listname);
 	my $curfext = $file_ext[-1];
         if (@Omit) {
-            foreach (@Omit) { return OK if ($curfext eq $_); }
+            foreach (@Omit) { return OK if (lc($curfext) eq $_); }
         }
         if (@Only) {
             my $skip = 1;
-            foreach (@Only) { $skip = 0 if ($curfext eq $_); }
+            foreach (@Only) { $skip = 0 if (lc($curfext) eq $_); }
             return OK if ($skip > 0);
         }
 	# Set the icon for the file
