@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # Pick a course
 #
-# $Id: lonpickcourse.pm,v 1.63.4.1 2007/08/02 19:53:54 albertel Exp $
+# $Id: lonpickcourse.pm,v 1.67.2.1 2007/11/28 17:36:11 albertel Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -72,7 +72,7 @@ sub handler {
     my $roledom = $env{'form.roleelement'};
     if ($roledom) {
         $roleelement = '<input type="hidden" name="roleelement" value="'.$roledom.'" />';
-        $submitopener = &Apache::lonroles::processpick();
+        $submitopener = &processpick();
         $autosubmit = 'process_pick("'.$roledom.'")';
     }
 
@@ -83,7 +83,8 @@ sub handler {
     }
 
     my %loaditem;
-    if ($env{'form.form'} eq 'cu' && $env{'form.pickedcourse'}) {
+    if ((($env{'form.form'} eq 'cu') || ($env{'form.form'} eq 'studentform')) && 
+        ($env{'form.pickedcourse'})) {
         $loaditem{'onload'} ="setSections()";
     }
     $r->print(&Apache::loncommon::start_page($title,undef,
@@ -92,7 +93,8 @@ sub handler {
 
     if ($env{'form.form'} eq 'portform') {
         $lastaction = 'document.courselist.submit()';
-    } elsif ($env{'form.form'} eq 'cu') {
+    } elsif ($env{'form.form'} eq 'cu' || ($env{'form.form'} eq 'studentform' &&
+        !$multiple)) {
         $lastaction = 
              'document.courselist.pickedcourse.value = cdom+"_"+cname;'."\n".
              'document.courselist.submit();';
@@ -101,7 +103,7 @@ sub handler {
     }
 
     # if called to assign a role in a course to a user via CUSR
-    if ($env{'form.form'} eq 'cu') {
+    if ($env{'form.form'} eq 'cu' || $env{'form.form'} eq 'studentform') {
         $r->print(&create_user_javascript($type)); 
     }
 
@@ -137,6 +139,49 @@ sub handler {
     return OK;
 }
 
+sub processpick {
+    my $openerform = 'rolechoice';
+    if ($env{'form.form'} eq 'studentform') {
+        $openerform = $env{'form.form'};
+    }
+    my $process_pick = <<"ENDONE";
+<script type="text/javascript">
+function process_pick(dom) {
+    var pickedCourse=opener.document.$openerform.$env{'form.cnumelement'}.value;
+    var pickedDomain=opener.document.$openerform.$env{'form.cdomelement'}.value;
+    var okDomain = 0;
+ENDONE
+    if ($openerform eq 'rolechoice') {
+        $process_pick .= <<"ENDTWO";
+    if (pickedDomain == dom) {
+        if (pickedCourse != '') {
+            var courseTarget = "cc./"+pickedDomain+"/"+pickedCourse
+            opener.document.title='Role selected. Please stand by.';
+            opener.status='Role selected. Please stand by.';
+            opener.document.rolechoice.newrole.value=courseTarget
+            opener.document.rolechoice.submit();
+        }
+    } 
+    else {
+        alert("You may only use this screen to select courses in the current domain: "+dom+"\\nPlease return to the roles page window and click the 'Select Course' link for domain: "+pickedDomain+",\\n if you are a Domain Coordinator in that domain, and wish to become a Course Coordinator in a course in the domain");
+    }
+ENDTWO
+    } else {
+        $process_pick .= <<"ENDTHREE";
+    if (pickedDomain != dom) {
+        alert("You may only use this screen to select courses in the current domain: "+dom+"\\nPlease return to the roles page window and click the 'Select Course' link for domain: "+pickedDomain+",\\n if you are a Domain Coordinator in that domain, and wish to become a Course Coordinator in a course in the domain");
+        return;
+    }
+ENDTHREE
+    }
+    $process_pick .= "
+}
+
+</script>
+";
+    return $process_pick;
+}
+
 sub create_user_javascript {
     my ($type) = @_;
     my $output;
@@ -175,7 +220,9 @@ sub display_matched_courses {
     my $numcourses = keys(%courses);
     foreach my $course (keys(%courses)) {
         my $descr;
-        if ($courses{$course} =~ m/^([^:]*):/i) {
+        if (ref($courses{$course}) eq 'HASH') {
+            $descr = $courses{$course}{'description'};
+        } elsif ($courses{$course} =~ m/^([^:]*):/i) {
             $descr = &unescape($1);
         } else {
             $descr = &unescape($courses{$course});
@@ -195,18 +242,36 @@ sub display_matched_courses {
             my $cleandesc=&HTML::Entities::encode($description,'<>&"');
             $cleandesc=~s/'/\\'/g;
             my ($cdom,$cnum)=split(/\_/,$course);
-            my ($descr,$instcode,$owner,$ttype) = split/:/,$courses{$course};
+            my ($descr,$instcode,$ttype,@owners);
+            if (ref($courses{$course}) eq 'HASH') {
+                $descr = $courses{$course}{'description'};
+                $instcode =  $courses{$course}{'inst_code'};
+                $ttype =  $courses{$course}{'type'};
+                push(@owners,&unescape($courses{$course}{'owner'}));
+                if (ref($courses{$course}{'co-owners'}) eq 'ARRAY') {
+                    foreach my $item (@{$courses{$course}{'co-owners'}}) {
+                        push(@owners,&unescape($item));
+                    }
+                }
+            } else {
+                my $singleowner; 
+                ($descr,$instcode,$singleowner,$ttype)=split(/:/,$courses{$course});
+                push(@owners,&unescape($singleowner));
+            }
+            my $owner = join(', ',@owners);
             $r->print(&course_chooser($multiple,$cdom,$cnum,$cleandesc));
             $r->print($description.'('.
                       (&Apache::lonnet::domain($cdom,'description')?
                        &Apache::lonnet::domain($cdom,'description'):$cdom).")");
-            unless ($instcode eq '') {
+            if ($instcode ne '') {
                 $r->print(" - ".&unescape($instcode));
             }
-            unless ($owner eq '') {
-                $r->print(", owner - ".&unescape($owner));
+            if (@owners > 1) {
+                $r->print(', '.&mt('owners').' - ',join(', ',@owners)); 
+            } elsif (@owners == 1) {
+                $r->print(', '.&mt('owner').' - '.$owner);
             }
-            unless ($ttype eq '') {
+            if ($ttype ne '') {
                 $r->print('('.&unescape($ttype).')');
             }
             if ($multiple) { $r->print("</label>\n"); }
@@ -283,7 +348,10 @@ sub build_filters {
 	    if ($formname eq 'portform') {
 		$filter->{$item} ||= $env{'user.domain'};
 		$allow_blank=0;
-	    }
+	    } elsif ($formname eq 'studentform') {
+                $filter->{$item} ||= $env{'request.role.domain'};
+                $allow_blank=0;
+            }
             $domainselectform =
 		&Apache::loncommon::select_dom_form($filter->{$item},
 						    'domainfilter',
@@ -322,8 +390,7 @@ sub build_filters {
               'cog' => &mt('Course Type')
              );
 
-    $typeselectform =  '<select name="type" size="1"';
-    $typeselectform .= ">\n";
+    $typeselectform =  '<select name="type" size="1">'."\n";
     if ($type eq 'Course') {
         $instcodeform = 
                 '<input type="text" name="instcodefilter" size="10" value="'.
@@ -425,7 +492,7 @@ sub search_courses {
         $r->print('<br />');
         my %coursehash = &Apache::loncommon::findallcourses(); 
         foreach my $cid (sort(keys(%coursehash))) {
-            $courses{$cid} = $env{'course.'.$cid.'.description'};
+            $courses{$cid}{'description'} = $env{'course.'.$cid.'.description'};
         }
     }
     return %courses;
@@ -689,7 +756,7 @@ Side Effects: None
 X<gochoose_javascript()>
 B<gochoose_javascript($type,$multiple,$autosubmit,$lastaction)>:
 
-Input: 4 - course type; single (0) or multiple courses (1); in context of DC selecting a CC role in a course: javascript code from Apache::lonroles::processpick(); final action to take after user chooses course(s):  either close window, or submit form for display of next page etc.
+Input: 4 - course type; single (0) or multiple courses (1); in context of DC selecting a CC role in a course: javascript code from &processpick(); final action to take after user chooses course(s):  either close window, or submit form for display of next page etc.
 
 Output: 1  $output - javascript wrapped in E<lt>scriptE<gt>E<lt>/scriptE<gt> tags
 
