@@ -1,6 +1,6 @@
 # The LearningOnline Network with CAPA
 #
-# $Id: lonstudentassessment.pm,v 1.144 2006/12/21 02:53:42 albertel Exp $
+# $Id: lonstudentassessment.pm,v 1.145 2007/10/03 17:39:13 albertel Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -1956,9 +1956,6 @@ sub student_tries_on_sequence {
             #
             if ( ($links eq 'yes' && $symbol ne ' ') ||
                  ($links eq 'all')) {
-                if (length($symbol) > 1) {
-                    &Apache::lonnet::logthis('length of symbol "'.$symbol.'" > 1');
-                }
                 my $link = '/adm/grades'.
                     '?symb='.&escape($resource->symb).
                         '&student='.$student->{'username'}.
