@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Localization routines
 #
-# $Id: lonlocal.pm,v 1.41 2006/08/28 14:26:25 albertel Exp $
+# $Id: lonlocal.pm,v 1.42 2007/10/18 21:08:31 albertel Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -164,6 +164,7 @@ use strict;
 use Apache::localize;
 use locale;
 use POSIX qw(locale_h strftime);
+use DateTime();
 
 require Exporter;
 
@@ -259,7 +260,10 @@ sub get_language_handle {
 # ========================================================== Localize localtime
 sub gettimezone {
     my ($time) = @_;
-    return ' ('.&strftime("%Z",localtime($time)).')';
+    if ($Apache::lonnet::env{'course.'.$Apache::lonnet::env{'request.course.id'}.'.timezone'}) {
+	return $Apache::lonnet::env{'course.'.$Apache::lonnet::env{'request.course.id'}.'.timezone'};
+    }
+    return 'local';
 }
 
 sub locallocaltime {
@@ -267,16 +271,26 @@ sub locallocaltime {
     if (!defined($thistime) || $thistime eq '') {
 	return &mt('Never');
     }
+
+    my $dt = DateTime->from_epoch(epoch => $thistime)
+                     ->set_time_zone(&gettimezone());
     if ((&current_language=~/^en/) || (!$lh)) {
-	return ''.localtime($thistime).&gettimezone($thistime);
+
+	return $dt->strftime("%a %b %e %I:%M:%S %P %Y (%Z)");
     } else {
 	my $format=$lh->maketext('date_locale');
 	if ($format eq 'date_locale') {
-	    return ''.localtime($thistime);
+	    return $dt->strftime("%a %b %e %I:%M:%S %P %Y (%Z)");
 	}
-	my ($seconds,$minutes,$twentyfour,$day,$mon,$year,$wday,$yday,$isdst)=
-	    localtime($thistime);
-	my $month=(split(/\,/,$lh->maketext('date_months')))[$mon];
+	my $time_zone  = $dt->time_zone_short_name();
+	my $seconds    = $dt->second();
+	my $minutes    = $dt->minute();
+	my $twentyfour = $dt->hour();
+	my $day        = $dt->day_of_month();
+	my $mon        = $dt->month()-1;
+	my $year       = $dt->year();
+	my $wday       = $dt->wday()-1;
+	my $month  =(split(/\,/,$lh->maketext('date_months')))[$mon];
 	my $weekday=(split(/\,/,$lh->maketext('date_days')))[$wday];
 	if ($seconds<10) {
 	    $seconds='0'.$seconds;
@@ -284,7 +298,6 @@ sub locallocaltime {
 	if ($minutes<10) {
 	    $minutes='0'.$minutes;
 	}
-	$year+=1900;
 	my $twelve=$twentyfour;
 	my $ampm;
 	if ($twelve>12) {
@@ -293,12 +306,11 @@ sub locallocaltime {
 	} else {
 	    $ampm=$lh->maketext('date_am');
 	}
-	foreach 
-	('seconds','minutes','twentyfour','twelve','day','year',
-	 'month','weekday','ampm') {
+	foreach ('seconds','minutes','twentyfour','twelve','day','year',
+		 'month','weekday','ampm') {
 	    $format=~s/\$$_/eval('$'.$_)/gse;
 	}
-	return $format.&gettimezone($thistime);
+	return $format." ($time_zone)";
     }
 }
 
