@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Dynamic plot
 #
-# $Id: lonplot.pm,v 1.118.2.3 2007/10/09 22:26:54 albertel Exp $
+# $Id: lonplot.pm,v 1.131 2007/10/09 22:11:51 albertel Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -36,11 +36,10 @@ use Apache::response;
 use Apache::lonxml;
 use Apache::edit;
 use Apache::lonnet;
-use lib '/home/httpd/lib/perl/';
 use LONCAPA;
  
 
-use vars qw/$weboutputformat $versionstring/;
+use vars qw/$weboutputformat $version/;
 
 
 
@@ -49,8 +48,9 @@ BEGIN {
     #
     # Determine the version of GNUPLOT
     $weboutputformat = 'gif';
-    $versionstring = `gnuplot --version 2>/dev/null`;
-    if ($versionstring =~ /^gnuplot 4/) {
+    my $versionstring = `gnuplot --version 2>/dev/null`;
+    ($version) = ($versionstring =~ /^gnuplot ([\d.]+)/);
+    if ($version >= 4) {
         $weboutputformat = 'png';
     }
     
@@ -117,7 +117,7 @@ my $pos_real_test  =
 my $color_test     = sub {$_[0]=~s/\s+//g;$_[0]=~/^x[\da-fA-F]{6}$/};
 my $onoff_test     = sub {$_[0]=~/^(on|off)$/};
 my $key_pos_test   = sub {$_[0]=~/^(top|bottom|right|left|outside|below| )+$/};
-my $sml_test       = sub {$_[0]=~/^(small|medium|large)$/};
+my $sml_test       = sub {$_[0]=~/^(\d+|small|medium|large)$/};
 my $linestyle_test = sub {exists($linestyles{$_[0]})};
 my $words_test     = sub {$_[0]=~s/\s+/ /g;$_[0]=~/^([\w~!\@\#\$\%^&\*\(\)-=_\+\[\]\{\}:\;\'<>,\.\/\?\\]+ ?)+$/};
 
@@ -127,8 +127,9 @@ my $words_test     = sub {$_[0]=~s/\s+/ /g;$_[0]=~/^([\w~!\@\#\$\%^&\*\(\)-=_\+\
 ##                                                               ##
 ###################################################################
 my @gnuplot_edit_order = 
-    qw/alttag bgcolor fgcolor height width font transparent grid samples 
-    border align texwidth texfont plotcolor plottype gridtype lmargin rmargin
+    qw/alttag bgcolor fgcolor height width texwidth fontface font texfont
+    transparent grid samples 
+    border align plotcolor plottype gridtype lmargin rmargin
     tmargin bmargin major_ticscale minor_ticscale boxwidth gridlayer fillstyle
     pattern solid/;
 
@@ -139,35 +140,35 @@ my %gnuplot_defaults =
      alttag       => {
 	 default     => 'dynamically generated plot',
 	 test        => $words_test,
-	 description => 'brief description of the plot',
+	 description => 'Brief description of the plot',
       	 edit_type   => 'entry',
 	 size        => '40'
 	 },
      height       => {
 	 default     => 300,
 	 test        => $int_test,
-	 description => 'height of image (pixels)',
+	 description => 'Height of image (pixels)',
       	 edit_type   => 'entry',
 	 size        => '10'
 	 },
      width        => {
 	 default     => 400,
 	 test        => $int_test,
-	 description => 'width of image (pixels)',
+	 description => 'Width of image (pixels)',
 	 edit_type   => 'entry',
 	 size        => '10'
 	 },
      bgcolor      => {
 	 default     => 'xffffff',
 	 test        => $color_test, 
-	 description => 'background color of image (xffffff)',
+	 description => 'Background color of image (xffffff)',
 	 edit_type   => 'entry',
 	 size        => '10'
 	 },
      fgcolor      => {
 	 default     => 'x000000',
 	 test        => $color_test,
-	 description => 'foreground color of image (x000000)',
+	 description => 'Foreground color of image (x000000)',
 	 edit_type   => 'entry',
 	 size        => '10'
 	 },
@@ -203,12 +204,19 @@ my %gnuplot_defaults =
 	 edit_type   => 'onoff'
 	 },
      font         => {
-	 default     => 'medium',
+	 default     => '9',
 	 test        => $sml_test,
-	 description => 'Size of font to use',
+	 description => 'Font size to use in web output (pts)',
 	 edit_type   => 'choice',
-	 choices     => ['small','medium','large']
+	 choices     => [['5','5 (small)'],'6','7','8',['9','9 (medium)'],'10',['11','11 (large)'],'12','15']
 	 },
+     fontface     => {
+        default     => 'sans-serif',
+        test        => sub {$_[0]=~/^(sans-serif|serif|classic)$/},
+        description => 'Type of font to use',
+        edit_type   => 'choice',
+        choices     => ['sans-serif','serif', 'classic']
+        },
      samples      => {
 	 default     => '100',
 	 test        => $int_test,
@@ -219,7 +227,7 @@ my %gnuplot_defaults =
      align        => {
 	 default     => 'middle',
 	 test        => sub {$_[0]=~/^(left|right|middle|center)$/},
-	 description => 'alignment for image in html',
+	 description => 'Alignment for image in HTML',
 	 edit_type   => 'choice',
 	 choices     => ['left','right','middle']
 	 },
@@ -247,7 +255,7 @@ my %gnuplot_defaults =
      pattern      => {
 	 default     => '',
 	 test        => $int_test,
-	 description => 'pattern value for boxes:',
+	 description => 'Pattern value for boxes:',
 	 edit_type   => 'choice',
          choices     => [0,1,2,3,4,5,6]
          },
@@ -310,7 +318,7 @@ my %gnuplot_defaults =
      boxwidth     => {
 	 default     => '',
 	 test        => $real_test, 
-	 description => 'width of boxes default auto',
+	 description => 'Width of boxes, default is auto',
 	 edit_type   => 'entry',
          size        => '5'
          },
@@ -348,7 +356,7 @@ my %key_defaults =
      pos   => { 
 	 default => 'top right', 
 	 test => $key_pos_test, 
-	 description => 'position of the key on the plot',
+	 description => 'Position of the key on the plot',
 	 edit_type   => 'choice',
 	 choices     => ['top left','top right','bottom left','bottom right',
 			 'outside','below']
@@ -360,14 +368,14 @@ my %label_defaults =
      xpos    => {
 	 default => 0,
 	 test => $real_test,
-	 description => 'x position of label (graph coordinates)',
+	 description => 'X position of label (graph coordinates)',
 	 edit_type   => 'entry',
 	 size        => '10'
 	 },
      ypos    => {
 	 default => 0, 
 	 test => $real_test,
-	 description => 'y position of label (graph coordinates)',
+	 description => 'Y position of label (graph coordinates)',
 	 edit_type   => 'entry',
 	 size        => '10'
 	 },
@@ -394,7 +402,7 @@ my %tic_defaults =
      mirror => {
 	 default => 'on', 
 	 test => $onoff_test,
-	 description => 'mirror tics on opposite axis?',
+	 description => 'Mirror tics on opposite axis?',
 	 edit_type   => 'onoff'
 	 },
      start => {
@@ -433,35 +441,35 @@ my %axis_defaults =
      color   => {
 	 default => 'x000000', 
 	 test => $color_test,
-	 description => 'color of grid lines (x000000)',
+	 description => 'Color of grid lines (x000000)',
 	 edit_type   => 'entry',
 	 size        => '10'
 	 },
      xmin      => {
 	 default => '-10.0',
 	 test => $real_test,
-	 description => 'minimum x-value shown in plot',
+	 description => 'Minimum x-value shown in plot',
 	 edit_type   => 'entry',
 	 size        => '10'
 	 },
      xmax      => {
 	 default => ' 10.0',
 	 test => $real_test,
-	 description => 'maximum x-value shown in plot',	 
+	 description => 'Maximum x-value shown in plot',	 
 	 edit_type   => 'entry',
 	 size        => '10'
 	 },
      ymin      => {
 	 default => '-10.0',
 	 test => $real_test,
-	 description => 'minimum y-value shown in plot',	 
+	 description => 'Minimum y-value shown in plot',	 
 	 edit_type   => 'entry',
 	 size        => '10'
 	 },
      ymax      => {
 	 default => ' 10.0',
 	 test => $real_test,
-	 description => 'maximum y-value shown in plot',	 
+	 description => 'Maximum y-value shown in plot',	 
 	 edit_type   => 'entry',
 	 size        => '10'
 	 }
@@ -474,14 +482,14 @@ my %curve_defaults =
      color     => {
 	 default => 'x000000',
 	 test => $color_test,
-	 description => 'color of curve (x000000)',
+	 description => 'Color of curve (x000000)',
 	 edit_type   => 'entry',
 	 size        => '10'
 	 },
      name      => {
 	 default => '',
 	 test => $words_test,
-	 description => 'name of curve to appear in key',
+	 description => 'Name of curve to appear in key',
 	 edit_type   => 'entry',
 	 size        => '20'
 	 },
@@ -502,21 +510,21 @@ my %curve_defaults =
      pointsize => {
          default     => 1,
          test        => $pos_real_test,
-         description => 'point size (may not apply to all line styles)',
+         description => 'Point size (may not apply to all line styles)',
          edit_type   => 'entry',
          size        => '5'
          },
      pointtype => {
          default     => 1,
          test        => $int_test,
-         description => 'point type (may not apply to all line styles)',
+         description => 'Point type (may not apply to all line styles)',
          edit_type   => 'choice',
          choices     => [0,1,2,3,4,5,6]
          },
      limit     => {
          default     => 'closed',
 	 test        => sub {$_[0]=~/^(closed|x1|x2|y1|y2)$/},
-         description => 'point to fill -- for filledcurves',
+         description => 'Point to fill -- for filledcurves',
          edit_type   => 'choice',
          choices     => ['closed','x1','x2','y1','y2']
          },
@@ -673,6 +681,51 @@ sub end_ytics {
     return $result;
 }
 
+##-----------------------------------------------------------------font
+my %font_properties =
+    (
+     'classic'    => {
+	 face       => 'classic',
+	 file       => 'DejaVuSansMono-Bold',
+	 printname  => 'Helvetica',
+	 tex_no_file => 1,
+     },
+     'sans-serif' => {
+	 face       => 'sans-serif',
+	 file       => 'DejaVuSans',
+	 printname  => 'DejaVuSans',
+     },
+     'serif'      => {
+	 face       => 'serif',
+	 file       => 'DejaVuSerif',
+	 printname  => 'DejaVuSerif',
+     },
+     );
+
+sub get_font {
+    my ($target) = @_;
+    my ($size, $selected_font);
+
+    if ( $Apache::lonplot::plot{'font'} =~ /^(small|medium|large)/) {
+	$selected_font = $font_properties{'classic'};
+	if ( $Apache::lonplot::plot{'font'} eq 'small') {
+	    $size = '5';
+	} elsif ( $Apache::lonplot::plot{'font'} eq 'medium') {
+	    $size = '9';
+	} elsif ( $Apache::lonplot::plot{'font'} eq 'large') {
+	    $size = '11';
+	} else {
+	    $size = '9';
+	}
+    } else {
+	$size = $Apache::lonplot::plot{'font'};
+	$selected_font = $font_properties{$Apache::lonplot::plot{'fontface'}};
+    }
+    if ($target eq 'tex' && defined($Apache::lonplot::plot{'texfont'})) {
+	$size = $Apache::lonplot::plot{'texfont'};
+    }
+    return ($size, $selected_font);
+}
 
 ##----------------------------------------------------------------- key
 sub start_key {
@@ -704,6 +757,47 @@ sub end_key {
     return $result;
 }
 
+sub parse_label {
+    my ($target,$text) = @_;
+    my $parser=HTML::LCParser->new(\$text);
+    my $result;
+    while (my $token=$parser->get_token) {
+	if ($token->[0] eq 'S') {
+	    if ($token->[1] eq 'sub') {
+		$result .= '_{';
+	    } elsif ($token->[1] eq 'sup') {
+		$result .= '^{';
+	    } else {
+		$result .= $token->[4];
+	    }
+	} elsif ($token->[0] eq 'E') {
+	    if ($token->[1] eq 'sub'
+		|| $token->[1] eq 'sup') {
+		$result .= '}';
+	    } else {
+		$result .= $token->[2];
+	    }
+	} elsif ($token->[0] eq 'T') {
+	    $result .= &replace_entities($target,$token->[1]);
+	}
+    }
+    return $result;
+}
+
+
+my %lookup = 
+    ('(pi|#960)' => {'tex' => '{/Symbol p}', 'web' => "\x{3C0}"},);
+
+sub replace_entities {
+    my ($target,$text) = @_;
+    $text =~ s{([_^~\{\}]|\\\\)}{\\\\$1}g;
+    while (my ($re, $replace) = each(%lookup)) {
+	$text =~ s/&$re;/$replace->{$target}/g;
+    }
+    $text =~ s{(&)}{\\\\$1}g;
+    return $text;
+}
+
 ##------------------------------------------------------------------- title
 sub start_title {
     my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
@@ -715,6 +809,7 @@ sub start_title {
 	if (length($title) > $max_str_len) {
 	    $title = substr($title,0,$max_str_len);
 	}
+	$title = &parse_label($target,$title);
     } elsif ($target eq 'edit') {
 	$result.=&Apache::edit::tag_start($target,$token,'Plot Title');
 	my $text=&Apache::lonxml::get_all_text("/title",$parser,$style);
@@ -746,6 +841,7 @@ sub start_xlabel {
 	if (length($xlabel) > $max_str_len) {
 	    $xlabel = substr($xlabel,0,$max_str_len);
 	}
+	$xlabel = &parse_label($target,$xlabel);
     } elsif ($target eq 'edit') {
 	$result.=&Apache::edit::tag_start($target,$token,'Plot Xlabel');
 	my $text=&Apache::lonxml::get_all_text("/xlabel",$parser,$style);
@@ -778,6 +874,7 @@ sub start_ylabel {
 	if (length($ylabel) > $max_str_len) {
 	    $ylabel = substr($ylabel,0,$max_str_len);
 	}
+	$ylabel = &parse_label($target,$ylabel);
     } elsif ($target eq 'edit') {
 	$result .= &Apache::edit::tag_start($target,$token,'Plot Ylabel');
 	my $text = &Apache::lonxml::get_all_text("/ylabel",$parser,$style);
@@ -811,7 +908,7 @@ sub start_label {
 	$text = &Apache::run::evaluate($text,$safeeval,$$parstack[-1]);
 	$text =~ s/\n/ /g;
 	$text = substr($text,0,$max_str_len) if (length($text) > $max_str_len);
-	$label{'text'} = $text;
+	$label{'text'} = &parse_label($target,$text);
 	push(@labels,\%label);
     } elsif ($target eq 'edit') {
 	$result .= &Apache::edit::tag_start($target,$token,'Plot Label');
@@ -1068,9 +1165,9 @@ sub get_attributes{
 ##------------------------------------------------------- write_gnuplot_file
 sub write_gnuplot_file {
     my ($tmpdir,$filename,$target)= @_;
+    my ($fontsize, $font_properties) =  &get_font($target);
     my $gnuplot_input = '';
     my $curve;
-    my $pt = $Apache::lonplot::plot{'texfont'};
     #
     # Check to be sure we do not have any empty curves
     my @curvescopy;
@@ -1101,16 +1198,25 @@ sub write_gnuplot_file {
     }
     # set term
     if ($target eq 'web') {
-	$gnuplot_input .= 'set term '.$weboutputformat .' ';
+	$gnuplot_input .= 'set terminal png enhanced nocrop ';
 	$gnuplot_input .= 'transparent ' if ($Apache::lonplot::plot{'transparent'} eq 'on');
-	$gnuplot_input .= $Apache::lonplot::plot{'font'} . ' ';
-	$gnuplot_input .= 'size '.$Apache::lonplot::plot{'width'}.','.$Apache::lonplot::plot{'height'}.' ';
+	$gnuplot_input .= 'font "'.$Apache::lonnet::perlvar{'lonFontsDir'}.
+	    '/'.$font_properties->{'file'}.'.ttf" ';
+	$gnuplot_input .= $fontsize;
+	$gnuplot_input .= ' size '.$Apache::lonplot::plot{'width'}.','.$Apache::lonplot::plot{'height'}.' ';
 	$gnuplot_input .= "@Colors\n";
 	# set output
 	$gnuplot_input .= "set output\n";
     } elsif ($target eq 'tex') {
-	$gnuplot_input .= "set term postscript eps $Apache::lonplot::plot{'plotcolor'} solid \"Helvetica\" $pt \n";
-	$gnuplot_input .= "set output \"/home/httpd/perl/tmp/".
+	$gnuplot_input .= "set term postscript eps enhanced $Apache::lonplot::plot{'plotcolor'} solid ";
+	if (!$font_properties->{'tex_no_file'}) {
+	    $gnuplot_input .=
+		'fontfile "'.$Apache::lonnet::perlvar{'lonFontsDir'}.
+		'/'.$font_properties->{'file'}.'.pfb" ';
+	}
+	$gnuplot_input .= ' "'.$font_properties->{'printname'}.'" ';
+	$gnuplot_input .= $fontsize;
+	$gnuplot_input .= "\nset output \"/home/httpd/perl/tmp/".
 	    &unescape($filename).".eps\"\n";
     }
     # cartesian or polar plot?
@@ -1153,9 +1259,15 @@ sub write_gnuplot_file {
     if (lc($Apache::lonplot::plot{'bmargin'}) ne 'default') {
         $gnuplot_input .= 'set bmargin '.$Apache::lonplot::plot{'bmargin'}.$/;
     }
+
     # tic scales
-    $gnuplot_input .= 'set ticscale '.
-        $Apache::lonplot::plot{'major_ticscale'}.' '.$Apache::lonplot::plot{'minor_ticscale'}.$/;
+    if ($version > 4) {
+	$gnuplot_input .= 'set tics scale '.
+	    $Apache::lonplot::plot{'major_ticscale'}.', '.$Apache::lonplot::plot{'minor_ticscale'}.$/;
+    } else {
+    	$gnuplot_input .= 'set ticscale '.
+	    $Apache::lonplot::plot{'major_ticscale'}.' '.$Apache::lonplot::plot{'minor_ticscale'}.$/;
+    }
     #boxwidth
     if (lc($Apache::lonplot::plot{'boxwidth'}) ne '') {
 	$gnuplot_input .= 'set boxwidth '.$Apache::lonplot::plot{'boxwidth'}.$/;
@@ -1174,14 +1286,17 @@ sub write_gnuplot_file {
     $gnuplot_input .= "set samples $Apache::lonplot::plot{'samples'}\n";
     # title, xlabel, ylabel
     # titles
+    my $extra_space_x = ($xtics{'location'} eq 'axis') ? ' 0, -0.5 ' : '';
+    my $extra_space_y = ($ytics{'location'} eq 'axis') ? ' -0.5, 0 ' : '';
+
     if ($target eq 'tex') {
-        $gnuplot_input .= "set title  \"$title\" font \"Helvetica,".$pt."pt\"\n"  if (defined($title)) ;
-        $gnuplot_input .= "set xlabel \"$xlabel\" font \"Helvetica,".$pt."pt\" \n" if (defined($xlabel));
-        $gnuplot_input .= "set ylabel \"$ylabel\" font \"Helvetica,".$pt."pt\"\n" if (defined($ylabel));
+	$gnuplot_input .= "set title  \"$title\"          font \"".$font_properties->{'printname'}.",".$fontsize."pt\"\n" if (defined($title)) ;
+	$gnuplot_input .= "set xlabel \"$xlabel\" $extra_space_x font \"".$font_properties->{'printname'}.",".$fontsize."pt\"\n" if (defined($xlabel));
+	$gnuplot_input .= "set ylabel \"$ylabel\" $extra_space_y font \"".$font_properties->{'printname'}.",".$fontsize."pt\"\n" if (defined($ylabel));
     } else {
-        $gnuplot_input .= "set title  \"$title\"  \n"  if (defined($title)) ;
-        $gnuplot_input .= "set xlabel \"$xlabel\" \n" if (defined($xlabel));
-        $gnuplot_input .= "set ylabel \"$ylabel\" \n" if (defined($ylabel));
+        $gnuplot_input .= "set title  \"$title\"          \n" if (defined($title)) ;
+        $gnuplot_input .= "set xlabel \"$xlabel\" $extra_space_x \n" if (defined($xlabel));
+        $gnuplot_input .= "set ylabel \"$ylabel\" $extra_space_y \n" if (defined($ylabel));
     }
     # tics
     if (%xtics) {    
@@ -1225,14 +1340,14 @@ sub write_gnuplot_file {
 	$gnuplot_input .= 'set label "'.$label->{'text'}.'" at '.
 	    $label->{'xpos'}.','.$label->{'ypos'}.' '.$label->{'justify'};
         if ($target eq 'tex') {
-            $gnuplot_input .=' font "Helvetica,'.$pt.'pt"' ;
+	    $gnuplot_input .=' font "'.$font_properties->{'printname'}.','.$fontsize.'pt"' ;
         }
         $gnuplot_input .= $/;
     }
     if ($target eq 'tex') {
         $gnuplot_input .="set size 1,".$Apache::lonplot::plot{'height'}/$Apache::lonplot::plot{'width'}*1.38;
         $gnuplot_input .="\n";
-        }
+    }
     # curves
     $gnuplot_input .= 'plot ';
     for (my $i = 0;$i<=$#curves;$i++) {
@@ -1278,7 +1393,7 @@ sub write_gnuplot_file {
 	    }
 	    #   write file
 	    print $fh $datatext;
-	    close ($fh);
+	    close($fh);
 	    #   generate gnuplot text
 	    $gnuplot_input.= '"'.$datafilename.'" title "'.
 		$curve->{'name'}.'" with '.
@@ -1298,7 +1413,8 @@ sub write_gnuplot_file {
 	}
     }
     # Write the output to a file.
-    my $fh=Apache::File->new(">$tmpdir$filename.data");
+    open (my $fh,">$tmpdir$filename.data");
+    binmode($fh, ":utf8");
     print $fh $gnuplot_input;
     close($fh);
     # That's all folks.
