@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # XML Parser Module 
 #
-# $Id: lonxml.pm,v 1.449.2.1 2007/09/05 01:03:21 albertel Exp $
+# $Id: lonxml.pm,v 1.469 2007/11/13 22:16:10 albertel Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -92,8 +92,10 @@ use Apache::loncacc();
 use Apache::lonmaxima();
 use Apache::lonlocal;
 
-#==================================================   Main subroutine: xmlparse  
+#====================================   Main subroutine: xmlparse  
+
 #debugging control, to turn on debugging modify the correct handler
+
 $Apache::lonxml::debug=0;
 
 # keeps count of the number of warnings and errors generated in a parse
@@ -135,6 +137,16 @@ $Apache::lonxml::request='';
 $Apache::lonxml::counter=1;
 $Apache::lonxml::counter_changed=0;
 
+# Part counter hash.   In analysis mode, the
+# problems can use this to record which parts increment the counter
+# by how much.  The counter subs will maintain this hash via
+# their optional part parameters.  Note that the assumption is that
+# analysis is done in one request and therefore it is not necessary to
+# save this information request-to-request.
+
+
+%Apache::lonxml::counters_per_part = ();
+
 #internal check on whether to look at style defs
 $Apache::lonxml::usestyle=1;
 
@@ -365,7 +377,7 @@ sub xmlparse {
      $finaloutput .= join('',@script_var_displays);
      undef(@script_var_displays);
  }
-
+ &init_state();
  if ($env{'form.return_only_error_and_warning_counts'}) {
      return "$errorcount:$warningcount";
  }
@@ -619,6 +631,24 @@ sub callsub {
   return $currentstring;
 }
 
+{
+    my %state;
+
+    sub init_state {
+	undef(%state);
+    }
+    
+    sub set_state {
+	my ($key,$value) = @_;
+	$state{$key} = $value;
+	return $value;
+    }
+    sub get_state {
+	my ($key) = @_;
+	return $state{$key};
+    }
+}
+
 sub setup_globals {
   my ($request,$target)=@_;
   $Apache::lonxml::request=$request;
@@ -627,6 +657,9 @@ sub setup_globals {
   $Apache::lonxml::default_homework_loaded=0;
   $Apache::lonxml::usestyle=1;
   &init_counter();
+  &clear_bubble_lines_for_part();
+  &init_state();
+  &set_state('target',$target);
   @Apache::lonxml::pwd=();
   @Apache::lonxml::extlinks=();
   @script_var_displays=();
@@ -800,6 +833,7 @@ sub init_safespace {
   $safehole->wrap(\&Math::Random::random_set_seed_from_phrase,$safeeval,'&random_set_seed_from_phrase');
   $safehole->wrap(\&Math::Random::random_get_seed,$safeeval,'&random_get_seed');
   $safehole->wrap(\&Math::Random::random_set_seed,$safeeval,'&random_set_seed');
+  $safehole->wrap(\&Apache::loncommon::languages,$safeeval,'&languages');
   $safehole->wrap(\&Apache::lonxml::error,$safeeval,'&LONCAPA_INTERNAL_ERROR');
   $safehole->wrap(\&Apache::lonxml::debug,$safeeval,'&LONCAPA_INTERNAL_DEBUG');
   $safehole->wrap(\&Apache::lonnet::logthis,$safeeval,'&LONCAPA_INTERNAL_LOGTHIS');
@@ -908,6 +942,9 @@ sub endredirection {
     }
     pop @Apache::lonxml::outputstack;
 }
+sub in_redirection {
+    return ($Apache::lonxml::redirection > 0)
+}
 
 sub end_tag {
   my ($tagstack,$parstack,$token)=@_;
@@ -1005,8 +1042,15 @@ sub get_all_text_unbalanced {
 	}
     }
     return $result
+
 }
 
+#########################################################################
+#                                                                       #
+#           bubble line counter management                              #
+#                                                                       #
+#########################################################################
+
 =pod
 
 For bubble grading mode and exam bubble printing mode, the tracking of
@@ -1022,16 +1066,31 @@ Increments the internal counter environment variable a specified amount
 
 Optional Arguments:
   $increment - amount to increment by (defaults to 1)
+               Also 1 if the value is negative or zero.
+  $part_response - A concatenation of the part and response id
+                   identifying exactly what is being 'answered'.
+
 
 =cut
 
 sub increment_counter {
-    my ($increment) = @_;
-    if (defined($increment) && $increment gt 0) {
-	$Apache::lonxml::counter+=$increment;
-    } else {
-	$Apache::lonxml::counter++;
+    my ($increment, $part_response) = @_;
+    if (!defined($increment) || $increment le 0) {
+	$increment = 1;
     }
+    $Apache::lonxml::counter += $increment;
+
+    # If the caller supplied the response_id parameter, 
+    # Maintain its counter.. creating if necessary.
+
+    if(defined($part_response)) {
+	if (!defined($Apache::lonxml::counters_per_part{$part_response})) {
+	    $Apache::lonxml::counters_per_part{$part_response} = 0;
+	}
+	$Apache::lonxml::counters_per_part{$part_response} += $increment;
+	my $new_value = $Apache::lonxml::counters_per_part{$part_response};
+    }
+	
     $Apache::lonxml::counter_changed=1;
 }
 
@@ -1088,6 +1147,74 @@ sub store_counter {
     }
 }
 
+=pod
+
+=item  bubble_lines_for_part(part_response)
+
+Returns the number of lines required to get a response for
+$part_response (this is just $Apache::lonxml::counters_per_part{$part_response}
+
+=cut
+
+sub bubble_lines_for_part {
+    my ($part_response) = @_;
+
+    if (!defined($Apache::lonxml::counters_per_part{$part_response})) {
+	return 0;
+    } else {
+	return $Apache::lonxml::counters_per_part{$part_response};
+    }
+
+}
+
+=pod
+
+=item clear_bubble_lines_for_part
+
+Clears the hash of bubble lines per part.  If a caller
+needs to analyze several resources this should be called between
+resources to reset the hash for each problem being analyzed.
+
+=cut
+
+sub clear_bubble_lines_for_part {
+    undef(%Apache::lonxml::counters_per_part);
+}
+
+=pod
+
+=item set_bubble_lines(part_response, value)
+
+If there is a problem part, that for whatever reason
+requires bubble lines that are not
+the same as the counter increment, it can call this sub during
+analysis to set its hash value explicitly.
+
+=cut
+
+sub set_bubble_lines {
+    my ($part_response, $value) = @_;
+
+    $Apache::lonxml::counters_per_part{$part_response} = $value;
+}
+
+=pod
+
+=item get_bubble_line_hash
+
+Returns the current bubble line hash.  This is assumed to 
+be small so we return a copy
+
+
+=cut
+
+sub get_bubble_line_hash {
+    return %Apache::lonxml::counters_per_part;
+}
+
+
+#--------------------------------------------------
+
 sub get_all_text {
     my($tag,$pars,$style)= @_;
     my $gotfullstack=1;
@@ -1350,41 +1477,58 @@ SIMPLECONTENT
 
 
 sub inserteditinfo {
-      my ($result,$filecontents,$filetype)=@_;
+      my ($filecontents,$filetype)=@_;
       $filecontents = &HTML::Entities::encode($filecontents,'<>&"');
 #      my $editheader='<a href="#editsection">Edit below</a><hr />';
       my $xml_help = '';
       my $initialize='';
+      my $textarea_id = 'filecont';
+      my ($add_to_onload, $add_to_onresize);
+      $initialize=&Apache::lonhtmlcommon::spellheader();
+      if ($filetype eq 'html' 
+	  && (!&Apache::lonhtmlcommon::htmlareablocked() &&
+	      &Apache::lonhtmlcommon::htmlareabrowser())) {
+	  $textarea_id .= '___Frame';
+	  my $lang = &Apache::lonhtmlcommon::htmlarea_lang();
+	  $initialize.=(<<FULLPAGE);
+<script type="text/javascript">
+lonca
+    function initDocument() {
+        var oFCKeditor = new FCKeditor('filecont');
+	oFCKeditor.Config['CustomConfigurationsPath'] = '/fckeditor/loncapaconfig.js'  ;
+	oFCKeditor.Config['FullPage'] = true
+	oFCKeditor.Config['AutoDetectLanguage'] = false;
+        oFCKeditor.Config['DefaultLanguage'] = "$lang";
+	oFCKeditor.ReplaceTextarea();
+    }
+    function check_if_dirty(editor) {
+	if (editor.IsDirty()) {
+	    unClean();
+	}
+    }
+    function FCKeditor_OnComplete(editor) {
+	editor.Events.AttachEvent("OnSelectionChange",check_if_dirty);
+	resize_textarea('$textarea_id','LC_aftertextarea');
+    }
+</script>
+FULLPAGE
+      } else {
+	  $initialize.=(<<FULLPAGE);
+<script type="text/javascript">
+    function initDocument() {
+	resize_textarea('$textarea_id','LC_aftertextarea');
+    }
+</script>
+FULLPAGE
+      }
+
+      $add_to_onload = 'initDocument();';
+      $add_to_onresize = "resize_textarea('$textarea_id','LC_aftertextarea');";
+
       if ($filetype eq 'html') {
-	  my $addbuttons=&Apache::lonhtmlcommon::htmlareaaddbuttons();
-	  $initialize=&Apache::lonhtmlcommon::spellheader();
-	  if (!&Apache::lonhtmlcommon::htmlareablocked() &&
-	      &Apache::lonhtmlcommon::htmlareabrowser()) {
-	      $initialize.=(<<FULLPAGE);
-<script type="text/javascript">
-$addbuttons
-
-    HTMLArea.loadPlugin("FullPage");
-
-    function initDocument() {
-	var editor=new HTMLArea("filecont",config);
-	editor.registerPlugin(FullPage);
-	editor.generate();
-    }
-</script>
-FULLPAGE
-          } else {
-	      $initialize.=(<<FULLPAGE);
-<script type="text/javascript">
-$addbuttons
-    function initDocument() {
-    }
-</script>
-FULLPAGE
-	  }
-          $result=~s/\<body([^\>]*)\>/\<body onload="initDocument()" $1\>/i;
 	  $xml_help=&Apache::loncommon::helpLatexCheatsheet();
       }
+
       my $cleanbut = '';
 
       my $titledisplay=&display_title();
@@ -1396,29 +1540,31 @@ FULLPAGE
       my $buttons=(<<BUTTONS);
 $cleanbut
 <input type="submit" name="discardview" accesskey="d"  value="$lt{'dv'}" />
-<input type="submit" name="Undo" accesskey="u"  value="$lt{'un'}" /><hr>
+<input type="submit" name="Undo" accesskey="u"  value="$lt{'un'}" /><hr />
 <input type="submit" name="savethisfile" accesskey="s"  value="$lt{'st'}" />
 <input type="submit" name="viewmode" accesskey="v" value="$lt{'vi'}" />
 BUTTONS
       $buttons.=&Apache::lonhtmlcommon::spelllink('xmledit','filecont');
+      my $textarea_events = &Apache::edit::element_change_detection();
+      my $form_events     = &Apache::edit::form_change_detection();
       my $editfooter=(<<ENDFOOTER);
 $initialize
 <hr />
 <a name="editsection" />
-<form method="post" name="xmledit">
+<form $form_events method="post" name="xmledit">
 $xml_help
 <input type="hidden" name="editmode" value="$lt{'ed'}" />
 $buttons<br />
-<textarea style="width:100%" cols="80" rows="44" name="filecont" id="filecont">$filecontents</textarea>
+<textarea $textarea_events style="width:100%" cols="80" rows="44" name="filecont" id="filecont">$filecontents</textarea>
+<div id="LC_aftertextarea">
 <br />$buttons
 <br />
-</form>
 $titledisplay
+</div>
+</form>
 </body>
 ENDFOOTER
-#      $result=~s/(\<body[^\>]*\>)/$1$editheader/is;
-      $result=~s/(\<\/body\>)/$editfooter/is;
-      return $result;
+      return ($editfooter,$add_to_onload,$add_to_onresize);;
 }
 
 sub get_target {
@@ -1524,32 +1670,54 @@ ENDNOTFOUND
 	    &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},
 						    ['rawmode']);
 	    if ($env{'form.rawmode'}) { $result = $filecontents; }
+	    if ($filetype eq 'sty') {
+		my $controls =
+		    ($env{'request.state'} eq 'construct') ? &Apache::londefdef::edit_controls()
+		                                           : '';
+		my %options = ('bgcolor' => '#FFFFFF');
+		$result = 
+		    &Apache::loncommon::start_page(undef,undef,\%options).
+		    $controls.
+		    $result.
+		    &Apache::loncommon::end_page();
+	    }
 	}
     }
-    
+
 #
 # Edit action? Insert editing commands
 #
     unless ($env{'request.state'} eq 'published') {
 	if ($env{'form.editmode'} && (!($env{'form.viewmode'})) && (!($env{'form.discardview'})))
-	    {
+	{
+	    my ($edit_info, $add_to_onload, $add_to_onresize)=
+		&inserteditinfo($filecontents,$filetype);
+
 	    my $displayfile=$request->uri;
 	    $displayfile=~s/^\/[^\/]*//;
-	    my %options = ();
+	    my %options = 
+		('add_entries' =>
+                   {'onresize' => $add_to_onresize,
+		    'onload'   => $add_to_onload,   });
+
 	    if ($env{'environment.remote'} ne 'off') {
 		$options{'bgcolor'}   = '#FFFFFF';
+		$options{'only_body'} = 1;
 	    }
-	    my $start_page = &Apache::loncommon::start_page(undef,undef,
+	    my $js =
+		&Apache::edit::js_change_detection().
+		&Apache::loncommon::resize_textarea_js();
+	    my $start_page = &Apache::loncommon::start_page(undef,$js,
 							    \%options);
 	    $result=$start_page.
 		&Apache::lonxml::message_location().'<h3>'.
 		$displayfile.
-		'</h3>'.&Apache::loncommon::end_page();
-	    $result=&inserteditinfo($result,$filecontents,$filetype);
+		'</h3>'.
+		$edit_info.
+		&Apache::loncommon::end_page();
 	}
     }
     if ($filetype eq 'html') { &writeallows($request->uri); }
-	
     
     &Apache::lonxml::add_messages(\$result);
     $request->print($result);
@@ -1598,12 +1766,28 @@ sub show_error_warn_msg {
 }
 
 sub error {
+    my @errors = @_;
+
     $errorcount++;
+
+    if (defined($Apache::inputtags::part)) {
+	if ( @Apache::inputtags::response ) {
+	    push(@errors,
+		 &mt("This error occurred while processing response [_1] in part [_2]",
+		     $Apache::inputtags::response[-1],
+		     $Apache::inputtags::part));
+	} else {
+	    push(@errors,
+		 &mt("This error occurred while processing part [_1]",
+		     $Apache::inputtags::part));
+	}
+    }
+
     if ( &show_error_warn_msg() ) {
 	# If printing in construction space, put the error inside <pre></pre>
 	push(@Apache::lonxml::error_messages,
 	     $Apache::lonxml::warnings_error_header.
-	     "<b>ERROR:</b>".join("<br />\n",@_)."<br />\n");
+	     "<b>ERROR:</b>".join("<br />\n",@errors)."<br />\n");
 	$Apache::lonxml::warnings_error_header='';
     } else {
 	my $errormsg;
@@ -1613,7 +1797,10 @@ sub error {
 	    $errormsg=&mt("An error occured while processing this resource. The author has been notified.");
 	}
 	my $host=$Apache::lonnet::perlvar{'lonHostID'};
-	my $msg = join('<br />',(@_,"The error occurred on host <tt>$host</tt>"));
+	push(@errors, "The error occurred on host <tt>$host</tt>");
+
+	my $msg = join('<br />', @errors);
+
 	#notify author
 	&Apache::lonmsg::author_res_msg($env{'request.filename'},$msg);
 	#notify course
